#ifndef __KEYLISTINTMD_H__
#define __KEYLISTINTMD_H__

#include "exttypes.h"
#include <geommath/geommath.h>
#include <stdio.h>

// GetNumKeys(), it returns the array length
// GetKey(index), it returns the key at the given index
// GetDuration(), it returns the amount of time it should take
// to play this particular animation.
// GetValue(t), returns a blended value
// keys reside in the area where t is between zero and one
// but any input to the function works.

class CKey1ListIntmd
{
	friend class CSceneCreatorIntmd;
	friend class CMeshInstanceIntmd;
	friend class CLightIntmd;
	friend class CCameraIntmd;
	
public:
	const int32 GetNumKeys() const { return m_iNumKeys; }
	const float GetKey(const int32 index) const { return m_pKeys[index]; }
	const float GetValue(const float t) const;
	const float GetDuration() const { return m_fDuration; }

	CKey1ListIntmd() : m_iNumKeys(0), m_pKeys(NULL), m_fDuration(-1) {}

private:
	int32 m_iNumKeys;
	float * m_pKeys;
	float m_fDuration;

private:	// internal
	void CleanUp();
};

class CKey3ListIntmd
{
	friend class CSceneCreatorIntmd;
	friend class CMeshInstanceIntmd;
	friend class CLightIntmd;
	friend class CCameraIntmd;
	
public:
	const int32 GetNumKeys() const { return m_iNumKeys; }
	const Vec3 GetKey(const int32 index) const { return m_pKeys[index]; }
	const Vec3 GetValue(const float t) const;
	const float GetDuration() const { return m_fDuration; }

	CKey3ListIntmd() : m_iNumKeys(0), m_pKeys(NULL), m_fDuration(-1) {}
	
private:
	int32 m_iNumKeys;
	Vec3 * m_pKeys;
	float m_fDuration;

private:	// internal
	void CleanUp();
};

class CQuatListIntmd
{
	friend class CSceneCreatorIntmd;
	friend class CMeshInstanceIntmd;
	friend class CLightIntmd;
	friend class CCameraIntmd;

public:
	const int32 GetNumKeys() const { return m_iNumKeys; }
	const Quat GetKey(const int32 index) const { return m_pKeys[index]; }
	const Quat GetValue(const float t) const;
	const float GetDuration() const { return m_fDuration; }

	CQuatListIntmd() : m_iNumKeys(0), m_pKeys(NULL), m_fDuration(-1) {}

private:
	int32 m_iNumKeys;
	Quat * m_pKeys;
	float m_fDuration;

private:	// internal
	void CleanUp();
};


#endif

