#ifndef __SCENECREATORINTMD_H__
#define __SCENECREATORINTMD_H__

#include "scenecreatorparse.h"


class CKey1ListIntmd;
class CSurfaceIntmd;
class CTextureIntmd;

class CSceneIntmd;

// intermediate level of the scene creator
class CSceneCreatorIntmd : public CSceneCreatorParse
{
public:
	virtual bool LoadToScene(const char scene[], const char geomdata[]);
	virtual CSceneIntmd& GetSceneIntmd() const;
	void CleanUp();

	// specific cleanup functions
	// used instead of cleanup
	// for people that want to preserve
	// animations kept internally by the loader
	void CleanUpMeshes();
	void CleanUpTextures();
	
	CSceneCreatorIntmd();
	~CSceneCreatorIntmd();

protected:
	virtual bool LoadToScene(IFileHandler * pFileSc, IFileHandler * pFileGeo);
	
	CSceneIntmd * m_pScene;

private:
	virtual void SceneBegin();
	virtual bool SceneEnd();

	virtual void SetAnimTime(const float fAnimTime);

	virtual void MaterialBegin(const char mat_name[]);
	virtual void MaterialEnd();
	virtual void DiffTextureBegin();
	virtual void DiffTextureEnd();
	virtual void ReflTextureBegin();
	virtual void ReflTextureEnd();
	virtual void BumpTextureBegin();
	virtual void BumpTextureEnd();
	virtual void SpecTextureBegin();
	virtual void SpecTextureEnd();

	virtual void MeshBegin(const char mesh_name[]);
	virtual void MeshEnd();
	virtual void SurfaceBegin(const int numverts, const int numstrips, const char matref[]);
	virtual void SurfaceEnd();
	virtual void StripBegin(const int len);
	virtual void StripEnd();
	virtual void AddVertex(const float x, const float y, const float z);
	virtual void AddNormal(const float nx, const float ny, const float nz);
	virtual void AddTangent(const float tx, const float ty, const float tz);
	virtual void AddBinormal(const float bx, const float by, const float bz);
	virtual void AddTexCoord(const float s, const float t);
	virtual void AddIndex(const int32 index);

	virtual void MeshinstanceBegin(const char meshinst_name[]);
	virtual void MeshinstanceEnd();
	virtual void SetMeshRef(const char meshref[]);

	virtual void CameraBegin(const char camera_name[]);
	virtual void CameraEnd();

	virtual void LightBegin(const char light_name[]);
	virtual void LightEnd();
	
	
	// optional userdefined durations of spline playtime
	// are set using a call to the SetDuration() function
	virtual void PositionBegin(const int32 array_len, const float fDuration=-1);
	virtual void PositionEnd();
	virtual void QuaternionBegin(const int32 array_len, const float fDuration=-1);
	virtual void QuaternionEnd();
	virtual void ScaleBegin(const int32 array_len, const float fDuration=-1);
	virtual void ScaleEnd();
	
	virtual void FieldOfViewBegin(const int32 array_len, const float fDuration=-1);
	virtual void FieldOfViewEnd();
	virtual void SetRatio(const float fAspect);

	virtual void Color3fBegin(const int32 array_len, const float fDuration=-1);
	virtual void Color3fEnd();
	virtual void NearBegin(const int32 array_len, const float fDuration=-1);
	virtual void NearEnd();
	virtual void FarBegin(const int32 array_len, const float fDuration=-1);
	virtual void FarEnd();
	virtual void HotSpotBegin(const int32 array_len, const float fDuration=-1);
	virtual void HotSpotEnd();
	virtual void FallOffBegin(const int32 array_len, const float fDuration=-1);
	virtual void FallOffEnd();
	virtual void AspectBegin(const int32 array_len, const float fDuration=-1);
	virtual void AspectEnd();
	virtual void SetSpot();
	virtual void SetConeRectangular();
	virtual void SetConeCircular();

	virtual void SetKey1(const int32 index, const float val);
	virtual void SetKey3(const int32 index, const float x, const float y, const float z);
	virtual void SetQuat(const int32 index, const float x, const float y, const float z, const float w);
	
	// bool is true if texture was already registered
	// in this case no texel data is to be delivered
	virtual bool TextureBegin(const uint32 uChkID);
	virtual void TextureEnd();
	virtual void MipmapBegin();
	virtual void MipmapEnd();
	virtual void SetTexDuration(const float fDuration);
	virtual void AddLatestTextureToSeq();
	virtual void SetUserDefProps(const char userdefprops[]);
	virtual void SetPixFormat(const int32 pix_format);
	virtual void SetColor(const int32 index, const uint8 r, const uint8 g, const uint8 b, const uint8 a);
	virtual void SetNumLevels(const int32 num_levels);
	virtual void SetDimension(const int32 width, const int32 height);
	virtual void SetTexel(const int32 x, const int32 y, const uint8 pen);
	virtual void SetTexel(const int32 x, const int32 y, const uint8 r, const uint8 g, const uint8 b, const uint8 a);

private:
	CTextureIntmd * m_pTexBuild;
	bool m_bAlreadyRegisteredTex;

private:
	enum INTMD_MODE	// modes for existing intermediate representations
	{
		BLANK_INTMD=0,
		SCENE_INTMD,
		MATERIAL_INTMD,
		MESH_INTMD,
		SURFACE_INTMD,
		STRIP_INTMD,
		MESHINSTANCE_INTMD,
		CAMERA_INTMD,
		LIGHT_INTMD,
		DIFFTEX_INTMD,
		REFLTEX_INTMD,
		BUMPTEX_INTMD,
		SPECTEX_INTMD,
		TEXTURE_INTMD,
		MIPMAP_INTMD,

		// animated types
		POSITION_INTMD,
		QUATERNION_INTMD,
		SCALE_INTMD,

		FIELDOFVIEW_INTMD,
		RATIO_INTMD,

		COLOR3F_INTMD,
		NEAR_INTMD,
		FAR_INTMD,
		HOTSPOT_INTMD,
		FALLOFF_INTMD,
		ASPECT_INTMD,

		// end
		NUM_INTMD_MODES
	};
	
	INTMD_MODE GetCurMode() { return m_eModeStack[m_iCurDepth]; }
	void PushMode(INTMD_MODE mode) { m_eModeStack[++m_iCurDepth] = mode; }
	void PopMode() { --m_iCurDepth; }

	int m_iCurDepth;
	INTMD_MODE m_eModeStack[NUM_INTMD_MODES];

private:	// data	
	// completion verifiers	(global tmp variables)
	static int32 vert_counter;
	static int32 normal_counter;
	static int32 tangent_counter;
	static int32 binormal_counter;
	static int32 st_counter;
	static int32 strip_counter;
	static int32 index_counter;
	static CSurfaceIntmd * pSurf;

	// more counters
	static int32 material_counter;
	static int32 mipmap_counter;
	static int32 mesh_counter;
	static int32 meshinstance_counter;

private:	// helper anim function
	void AllocKey1Array(CKey1ListIntmd * pKey1Array, const INTMD_MODE mode, const int32 array_len, const float fDuration);

private:	// anim time
	float m_fAnimTime;
};


#endif
