#ifndef __SCENECREATOR_H__
#define __SCENECREATOR_H__


#define MAX_TOKEN_STRING_LENGTH 25


#include "exttypes.h"
#include "filehandler.h"


class CSceneCreatorParse
{
public:
	bool LoadToScene(IFileHandler * pFileSc, IFileHandler * pFileGeo);

	CSceneCreatorParse();
	virtual ~CSceneCreatorParse();


	// Delivery functions
protected:
	virtual void SceneBegin()=0;
	virtual bool SceneEnd()=0;

	virtual void SetAnimTime(const float fAnimTime)=0;

	virtual void MaterialBegin(const char mat_name[])=0;
	virtual void MaterialEnd()=0;
	virtual void DiffTextureBegin()=0;
	virtual void DiffTextureEnd()=0;
	virtual void ReflTextureBegin()=0;
	virtual void ReflTextureEnd()=0;
	virtual void BumpTextureBegin()=0;
	virtual void BumpTextureEnd()=0;
	virtual void SpecTextureBegin()=0;
	virtual void SpecTextureEnd()=0;

	virtual void MeshBegin(const char mesh_name[])=0;
	virtual void MeshEnd()=0;
	virtual void SurfaceBegin(const int numverts, const int numstrips, const char matref[])=0;
	virtual void SurfaceEnd()=0;
	virtual void StripBegin(const int len)=0;
	virtual void StripEnd()=0;
	virtual void AddVertex(const float x, const float y, const float z)=0;
	virtual void AddNormal(const float nx, const float ny, const float nz)=0;
	virtual void AddTangent(const float tx, const float ty, const float tz)=0;
	virtual void AddBinormal(const float bx, const float by, const float bz)=0;
	virtual void AddTexCoord(const float s, const float t)=0;
	virtual void AddIndex(const int32 index)=0;

	virtual void MeshinstanceBegin(const char meshinst_name[])=0;
	virtual void MeshinstanceEnd()=0;
	virtual void SetMeshRef(const char meshref[])=0;

	virtual void CameraBegin(const char camera_name[])=0;
	virtual void CameraEnd()=0;

	virtual void LightBegin(const char light_name[])=0;
	virtual void LightEnd()=0;
	
	// optional userdefined durations of spline playtime
	// are set using a call to the SetDuration() function
	virtual void PositionBegin(const int32 array_len, const float fDuration=-1)=0;
	virtual void PositionEnd()=0;
	virtual void QuaternionBegin(const int32 array_len, const float fDuration=-1)=0;
	virtual void QuaternionEnd()=0;
	virtual void ScaleBegin(const int32 array_len, const float fDuration=-1)=0;
	virtual void ScaleEnd()=0;
	
	virtual void FieldOfViewBegin(const int32 array_len, const float fDuration=-1)=0;
	virtual void FieldOfViewEnd()=0;
	virtual void SetRatio(const float fAspect)=0;

	virtual void Color3fBegin(const int32 array_len, const float fDuration=-1)=0;
	virtual void Color3fEnd()=0;
	virtual void NearBegin(const int32 array_len, const float fDuration=-1)=0;
	virtual void NearEnd()=0;
	virtual void FarBegin(const int32 array_len, const float fDuration=-1)=0;
	virtual void FarEnd()=0;
	virtual void HotSpotBegin(const int32 array_len, const float fDuration=-1)=0;
	virtual void HotSpotEnd()=0;
	virtual void FallOffBegin(const int32 array_len, const float fDuration=-1)=0;
	virtual void FallOffEnd()=0;
	virtual void AspectBegin(const int32 array_len, const float fDuration=-1)=0;
	virtual void AspectEnd()=0;
	virtual void SetSpot()=0;
	virtual void SetConeRectangular()=0;
	virtual void SetConeCircular()=0;

	// bool is true if texture was already registered
	// in this case no texel data is to be delivered
	virtual bool TextureBegin(const uint32 uChkID)=0;
	virtual void TextureEnd()=0;
	virtual void MipmapBegin()=0;
	virtual void MipmapEnd()=0;
	virtual void SetTexDuration(const float fDuration)=0;
	virtual void AddLatestTextureToSeq()=0;
	virtual void SetUserDefProps(const char userdefprops[])=0;
	virtual void SetPixFormat(const int32 pix_format)=0;
	virtual void SetColor(const int32 index, const uint8 r, const uint8 g, const uint8 b, const uint8 a)=0;
	virtual void SetNumLevels(const int32 num_levels)=0;
	virtual void SetDimension(const int32 width, const int32 height)=0;
	virtual void SetTexel(const int32 x, const int32 y, const uint8 pen)=0;
	virtual void SetTexel(const int32 x, const int32 y, const uint8 r, const uint8 g, const uint8 b, const uint8 a)=0;

	virtual void SetKey1(const int32 index, const float val)=0;
	virtual void SetKey3(const int32 index, const float x, const float y, const float z)=0;
	virtual void SetQuat(const int32 index, const float x, const float y, const float z, const float w)=0;

private:	// token parsing	
	enum ASC_TOKEN
	{
		LESS_THEN=0,
		GREATER_THEN,
		LEFT_CURLY,
		RIGHT_CURLY,
		LEFT_BRACKET,
		RIGHT_BRACKET,
		COMMA,
		COLON,
		STRING,

		INTEGER,
		REAL_VALUE,

		ANIMTIME,
		MATERIAL,
		MESH,
		SURFACE,
		MESHINSTANCE,
		MESHREF,
		CAMERA,
		LIGHT,

		POSITION,
		POSITIONARRAY,
		QUATERNION,
		QUATERNIONARRAY,
		SCALE,
		SCALEARRAY,
		
		FIELDOFVIEW,
		FIELDOFVIEWARRAY,
		RATIO,

		COLOR,
		COLORARRAY,
		NEAR,
		NEARARRAY,
		FAR,
		FARARRAY,
		HOTSPOT,
		HOTSPOTARRAY,
		FALLOFF,
		FALLOFFARRAY,
		ASPECT,
		ASPECTARRAY,
		SPOTSHAPE,
		CIRCULAR,
		RECTANGULAR,

		DIFFUSE,
		REFLECTION,
		BUMPMAP,
		SPECULAR,


		// length of enum list
		NUM_OF_ASC_TOKENS,
		UNKNOWN = NUM_OF_ASC_TOKENS
	};

	bool ParseScene();
	bool ParseAnimTime();
	bool ParseMaterial();
	bool ParseMesh();
	bool ParseMeshInstance();
	bool ParseCamera();
	bool ParseLight();
	bool ParseTexSeq(const ASC_TOKEN type);
	bool ParseTexture(const char cID[]);

	// anim functions
	bool ParseKeys3(const ASC_TOKEN type);
	bool ParseQuats(const ASC_TOKEN type);
	bool ParseKeys1(const ASC_TOKEN type);

	void FetchKeys1(const int32 num_keys, const bool bKeysInBinChunk, const float fDefVal);	// assisting function

	// helper func
	uint32 LocateID(const char cID[]);


private:	// lexical analysis		
	static char token_strings[][MAX_TOKEN_STRING_LENGTH];
	static int32 szeScene;
	static char * pScene;
	static struct Token
	{
		ASC_TOKEN token_type;

		// string location in scene buffer
		int32 str_len;
		char * cptr;	// string is not terminated
	} token;

	bool match(ASC_TOKEN type);
	bool fetch_token();
	float token_to_real();
	int token_to_integer();
	int char_to_digit(const char ch);
	void RemoveWhiteSpace();

private:	// file handling
	static IFileHandler * pFileGeomData;	// binary
	static IFileHandler * pFileScene;	// ASCII
};

#endif
