#ifndef __SURFACEINTMD_H__
#define __SURFACEINTMD_H__

#include <stdio.h>
#include "textureintmd.h"
#include "exttypes.h"
#include <geommath/geommath.h>

// intermediate representations of the various scene components

class CMaterialIntmd
{
	friend class CSceneCreatorIntmd;
	friend class CSceneIntmd;

public:
	char * GetName() const { return m_pMatName; }
	CTextureSeqIntmd& GetDiffTexture() { return m_cDiffTexture; }
	CTextureSeqIntmd& GetReflTexture() { return m_cReflTexture; }
	CTextureSeqIntmd& GetBumpTexture() { return m_cBumpTexture; }
	CTextureSeqIntmd& GetSpecTexture() { return m_cSpecTexture; }

	CMaterialIntmd() : m_pMatName(NULL) {}
	
private:
	void SetName(const char name[]);
	bool operator !=(const CMaterialIntmd &mat);

	char * m_pMatName;

private:
	void DiffTexInit();
	void ReflTexInit();	
	void BumpTexInit();
	void SpecTexInit();	
	
	CTextureSeqIntmd m_cDiffTexture;
	CTextureSeqIntmd m_cReflTexture;
	CTextureSeqIntmd m_cBumpTexture;
	CTextureSeqIntmd m_cSpecTexture;

private:	// internal
	void CleanUp();
};

class CStripIntmd
{	
	friend class CSceneCreatorIntmd;
	friend class CSurfaceIntmd;

public:
	int32 GetArrayLen() const { return m_len; }
	int32 GetIndex(const int32 iIndexNum) const { return m_pIndices[iIndexNum]; }

	CStripIntmd() : m_len(0), m_pIndices(NULL) {}

private:
	void Setup(const int32 len);
	bool operator !=(const CStripIntmd &strip);
	
	int32 m_len;
	int32 * m_pIndices;

private:	// internal
	void CleanUp();
};

class CSceneIntmd;

class CSurfaceIntmd
{
	friend class CSceneCreatorIntmd;
	friend class CMeshIntmd;

public:
	// you can get the material from CSceneIntmd
	// calling GetMesh() with this index as arg
	int32 GetMaterialIndex() const;
	int32 GetNumStrips()	const { return m_iNumStrips; }
	int32 GetArrayLen() const	{ return m_iLenArrays; }
	Vec3& GetVertex(int32 iVertNum) const { return m_pVerts[iVertNum]; }
	Vec3& GetNormal(int32 iNormalNum) const { return m_pNormals[iNormalNum]; }
	Vec3& GetTangent(int32 iTangentNum) const { return m_pTangents[iTangentNum]; }
	Vec3& GetBinormal(int32 iBiNormNum) const { return m_pBiNormals[iBiNormNum]; }
	Vec3& GetST(int32 iSTNum) const { return m_pSTs[iSTNum]; }
	CStripIntmd& GetStrip(int32 iStripNum) const { return m_pStrips[iStripNum]; }
	
	CSurfaceIntmd()
	{
		m_iNumStrips = 0;
		m_pStrips = NULL;
		m_iLenArrays = 0;
		m_pVerts = NULL;
		m_pNormals = NULL;
		m_pTangents = NULL;
		m_pBiNormals = NULL;
		m_pSTs = NULL;

		m_pOwnerScene = NULL;
		m_pMaterialRef = NULL;
	}
	
private:
	bool operator !=(const CSurfaceIntmd &surf);
	void Setup(const int32 numverts, const int32 numstrips, const char matref[]);

	
	int32 m_iNumStrips;
	CStripIntmd * m_pStrips;

	int32 m_iLenArrays;
	Vec3 * m_pVerts;
	Vec3 * m_pNormals;
	Vec3 * m_pTangents;
	Vec3 * m_pBiNormals;
	Vec3 * m_pSTs;

private:	// internal
	void CleanUp();
	CSceneIntmd * m_pOwnerScene;
	char * m_pMaterialRef;
};


#endif
