#ifndef __TEXTUREINTMD_H__
#define __TEXTUREINTMD_H__

#include "exttypes.h"
#include <stdio.h>

// 32bit colors are always rgba with red as least significant byte
// 32bit is the only unindexed texel format (palette format included).

class CBinaryBitmapIntmd
{
	friend class CTextureIntmd;
	friend class CSceneCreatorIntmd;

public:
	int32 GetWidth() const { return m_iWidth; }
	int32 GetHeight() const { return m_iHeight; }
	void * GetTexelData() const { return m_pTexelData; }

	CBinaryBitmapIntmd() : m_iWidth(-1), m_iHeight(-1), m_pTexelData(NULL) {}


private:	
	int32 m_iWidth;
	int32 m_iHeight;
	uint8 * m_pTexelData;	// check pixelformat in CInternTexture
};

class CTextureIntmd
{
	friend class CSceneIntmd;
	friend class CSceneCreatorIntmd;
	
public:
	char * GetUserDefProps() const { return m_pUserDefProps; }
	int32 GetPixelFormat() const { return m_iPixelFormat; }
	void * GetPalette() const { return m_pInternPalette; }
	int32 GetNumLevels() const { return m_iNumLevels; }
	CBinaryBitmapIntmd& GetLevel(const int32 iLevelNum) const { return m_pMipMaps[iLevelNum]; }

	CTextureIntmd() : m_pUserDefProps(NULL), m_iPixelFormat(-1), m_pInternPalette(NULL), m_iNumLevels(-1), m_pMipMaps(NULL), m_uChkID(0) {}


private:
	bool operator !=(const CTextureIntmd &tex);
	
	char * m_pUserDefProps;

	int32 m_iPixelFormat;
	uint8 * m_pInternPalette;	// max 256*4 ubytes
	int32 m_iNumLevels;		// Number of mipmap levels	(BinaryBitmaps)
	CBinaryBitmapIntmd * m_pMipMaps;	// always at least one level

private:	// internal use
	void CleanUp();
	uint32 m_uChkID;	// ID chunk from file
};


class CTextureSeqIntmd
{
	friend class CMaterialIntmd;
	friend class CSceneCreatorIntmd;
	
public:
	float GetDuration() const { return m_fDuration; }
	int32 GetNumTexFrames() const { return m_iNumTexFrames; }

	// you can get the texture from CSceneIntmd
	// calling GetTexture() with this index as arg
	int32 GetTexFrameIndex(const int32 iNumTex) const { return m_pTexFrames[iNumTex]; }

	CTextureSeqIntmd() : m_fDuration(-1.0f), m_iNumTexFrames(0), m_pTexFrames(NULL) {}

private:
	bool operator !=(const CTextureSeqIntmd &texseq) const;
	void IncTexBuffer(const int32 iNewTexFrameEntry);
	
	float m_fDuration;
	int32 m_iNumTexFrames;
	
private:	// internal
	void CleanUp();
	int32 * m_pTexFrames;
};


#endif
