#include <stdio.h>
#include <string.h>
#include "p2lassert.h"
#include "intmd/filehandlerstd.h"


#ifdef SPEED_HANDLER_STD

void CFileHandlerStd::Read(void * ptr, uint32 uBSize)
{
	if(m_bValid)
	{
		memcpy(ptr, m_pData+m_iCurPos, uBSize);
		m_iCurPos += (int32) uBSize;
	}
}

int32 CFileHandlerStd::GetCurPos()
{
	return (m_bValid) ? m_iCurPos : -1;
}

void CFileHandlerStd::OffsetFromCur(const int32 iOffs)
{
	if(m_bValid)
		m_iCurPos += iOffs;
}

void CFileHandlerStd::LocateStart()
{
	if(m_bValid)
		m_iCurPos = 0;
}

int32 CFileHandlerStd::Size()
{
	return (m_bValid) ? m_iSize : -1;
}




CFileHandlerStd::CFileHandlerStd(const char path[])
{
	m_bValid=false;
	FILE * pFile = NULL;
	if((pFile=fopen(path, "rb"))==NULL)
	{
		printf("!!!!!!!!!!!! ERROR CAN'T OPEN FILE %s\n", path);
		assert(false);	// could be commented out?
	}
	else
	{
		m_pData = NULL;
		fseek(pFile, 0, SEEK_END);
		int32 end = ftell(pFile);
		fseek(pFile, 0, SEEK_SET);
		m_iSize = end - ftell(pFile);
		assert(m_iSize > 0);
		m_iCurPos = 0;
		fseek(pFile, 0, SEEK_SET);
		if((m_pData=new char[m_iSize]) != NULL)
		{	fread(m_pData, m_iSize, 1, pFile); m_bValid=true;	}
		else
		{
			fclose(pFile);
			printf("Memory allocation error in filehandlerT10k: %s", path);
		}
	}
}

CFileHandlerStd::~CFileHandlerStd()
{
	if(m_bValid) delete [] m_pData;
}


#else


void CFileHandlerStd::Read(void * ptr, uint32 uBSize)
{
	if(m_bValid)
		fread(ptr, uBSize, 1, m_pFile);
}

int32 CFileHandlerStd::GetCurPos()
{
	int32 res = -1;
	if(m_bValid)
		res = ftell(m_pFile);

	return res;
}

void CFileHandlerStd::OffsetFromCur(const int32 iOffs)
{
	if(m_bValid)
		fseek(m_pFile, iOffs, SEEK_CUR);
}

void CFileHandlerStd::LocateStart()
{
	if(m_bValid)
		fseek(m_pFile, 0, SEEK_SET);
}

int32 CFileHandlerStd::Size()
{
	int32 res = -1;
	if(m_bValid)
	{
		const int32 pos = GetCurPos();
		fseek(m_pFile, 0, SEEK_END);
		int32 end = GetCurPos();
		fseek(m_pFile, 0, SEEK_SET);
		res = end - GetCurPos();
		fseek(m_pFile, pos, SEEK_SET);
	}

	return res;
}




CFileHandlerStd::CFileHandlerStd(const char path[])
{
	if((this->m_pFile=fopen(path, "rb"))==NULL)
	{
		printf("!!!!!!!!!!!! ERROR CAN'T OPEN FILE %s\n", path);
		m_bValid=false;
		assert(false);	// could be commented out?
	}
	else
		m_bValid=true;
}

CFileHandlerStd::~CFileHandlerStd()
{
	if(m_bValid) fclose(m_pFile);
}

#endif
