#include "intmd/keylistintmd.h"

// same as GetValue on Key3List
const float CKey1ListIntmd::GetValue(const float t) const
{
	// verify we have keys available
	if(m_iNumKeys == 0) return 0;
	else if(m_iNumKeys == 1) return GetKey(0);

	// let's locate keys
	const float t_clmp = (t < 0) ? 0 : ((t > 1) ? 1 : t);
	const int32 iNumKeysSubOne = m_iNumKeys-1;

	int32 i2 = (int32) (t_clmp * iNumKeysSubOne);
	if(i2 == iNumKeysSubOne) --i2;		// too far, rewind	
	int32 i3 = i2+1;
	int32 i1 = (i2 == 0) ? i2 : i2-1;
	int32 i4 = (i3 == iNumKeysSubOne) ? i3 : i3+1;

	// data
	const float P1 = GetKey(i2);
	const float P2 = GetKey(i3);
	const float R1 = P2 - GetKey(i1);
	const float R2 = GetKey(i4) - P1;
	
	// generate interpolated result (Hermite)
	const float t1 = t*iNumKeysSubOne - i2;	// time, relative to chosen polynomial
	const float t2 = t1*t1;
	const float t3 = t1*t2;

	const float b = 3*t2 - 2*t3;
	const float d = t3 - t2;
	const float a = 1 - b;
	const float c = d - t2 + t1;
	
	return a*P1 + b*P2 + c*R1 + d*R2;
}

// same as GetValue on Key1List
const Vec3 CKey3ListIntmd::GetValue(const float t) const
{
	// verify we have keys available
	if(m_iNumKeys == 0) return Vec3(0,0,0);
	else if(m_iNumKeys == 1) return GetKey(0);

	// let's locate keys
	const float t_clmp = (t < 0) ? 0 : ((t > 1) ? 1 : t);
	const int32 iNumKeysSubOne = m_iNumKeys-1;

	int32 i2 = (int32) (t_clmp * iNumKeysSubOne);
	if(i2 == iNumKeysSubOne) --i2;		// too far, rewind	
	int32 i3 = i2+1;
	int32 i1 = (i2 == 0) ? i2 : i2-1;
	int32 i4 = (i3 == iNumKeysSubOne) ? i3 : i3+1;

	// data
	const Vec3 &P1 = GetKey(i2);
	const Vec3 &P2 = GetKey(i3);
	const Vec3 R1 = P2 - GetKey(i1);
	const Vec3 R2 = GetKey(i4) - P1;
	
	// generate interpolated result (Hermite)
	const float t1 = t*iNumKeysSubOne - i2;	// time, relative to chosen polynomial
	const float t2 = t1*t1;
	const float t3 = t1*t2;

	const float b = 3*t2 - 2*t3;
	const float d = t3 - t2;
	const float a = 1 - b;
	const float c = d - t2 + t1;
	
	return a*P1 + b*P2 + c*R1 + d*R2;
}

const Quat CQuatListIntmd::GetValue(const float t) const
{
	// verify we have keys available
	if(m_iNumKeys == 0) return Quat(Vec3(0,0,0), 1);
	else if(m_iNumKeys == 1) return GetKey(0);

	// let's locate keys
	const float t_clmp = (t < 0) ? 0 : ((t > 1) ? 1 : t);
	const int32 iNumKeysSubOne = m_iNumKeys-1;

	int32 i1 = (int32) (t_clmp * iNumKeysSubOne);
	if(i1 == iNumKeysSubOne) --i1;		// too far, rewind	
	int32 i2 = i1+1;

	// data
	const Quat &Q1 = GetKey(i1);
	const Quat &Q2 = GetKey(i2);
	
	// generate interpolated result
	const float tLoc = t*iNumKeysSubOne - i1;	// time, relative to chosen keys
	
	return Slerp(Q1, Q2, tLoc);
}


// the clean up crew
void CKey1ListIntmd::CleanUp()
{
	m_fDuration = -1.0f;
	m_iNumKeys = 0;
	if(m_pKeys!=NULL) { delete [] m_pKeys; m_pKeys=NULL; }
}

void CKey3ListIntmd::CleanUp()
{
	m_fDuration = -1.0f;
	m_iNumKeys = 0;
	if(m_pKeys!=NULL) { delete [] m_pKeys; m_pKeys=NULL; }
}

void CQuatListIntmd::CleanUp()
{
	m_fDuration = -1.0f;
	m_iNumKeys = 0;
	if(m_pKeys!=NULL) { delete [] m_pKeys; m_pKeys=NULL; }
}
