#include <stdio.h>
#include "p2lassert.h"
#include "intmd/scenecreatorintmd.h"
#include "intmd/sceneintmd.h"
#include "intmd/textureintmd.h"
#include "intmd/filehandlerstd.h"
#include "intmd/pixel_formats.h"


// completion verifiers	(global tmp variables)
int32 CSceneCreatorIntmd::vert_counter;
int32 CSceneCreatorIntmd::normal_counter;
int32 CSceneCreatorIntmd::tangent_counter;
int32 CSceneCreatorIntmd::binormal_counter;
int32 CSceneCreatorIntmd::st_counter;
int32 CSceneCreatorIntmd::strip_counter;
int32 CSceneCreatorIntmd::index_counter;
CSurfaceIntmd * CSceneCreatorIntmd::pSurf;

// more counters
int32 CSceneCreatorIntmd::material_counter;
int32 CSceneCreatorIntmd::mipmap_counter;
int32 CSceneCreatorIntmd::mesh_counter;
int32 CSceneCreatorIntmd::meshinstance_counter;



// open data files
bool CSceneCreatorIntmd::LoadToScene(const char scene[], const char geomdata[])
{
	// replace the CFileHandlerStd if your platform of
	// choice doesn't support the standard I/O functions
	CFileHandlerStd sceneFile(scene), geomFile(geomdata);

	return CSceneCreatorIntmd::LoadToScene(&sceneFile, &geomFile);
}

bool CSceneCreatorIntmd::LoadToScene(IFileHandler * pFileSc, IFileHandler * pFileGeo)
{
	return CSceneCreatorParse::LoadToScene(pFileSc, pFileGeo);
}

CSceneIntmd& CSceneCreatorIntmd::GetSceneIntmd() const
{
	return *m_pScene;
}


// scene creation
void CSceneCreatorIntmd::SceneBegin()
{
	printf("scene begin\n");

	if(m_pScene == NULL)
	{
		if((m_pScene=new CSceneIntmd) == NULL)
		{
			assert(false);
		}
	}

	assert(GetCurMode() == BLANK_INTMD);
	PushMode(SCENE_INTMD);
	material_counter = 0;
	mesh_counter = 0;
	meshinstance_counter = 0;
}

bool CSceneCreatorIntmd::SceneEnd()
{
	printf("scene end\n");

	assert(GetCurMode() == SCENE_INTMD);
	PopMode();

	// colapse redundant materials due
	// to multiple scene files loaded
	if(material_counter > 0)
	{
		for(int m=material_counter-1; m>=0; m--)
		{
			bool bUnique = true;
			const int32 iOrgNumMaterials = m_pScene->GetNumMaterials()-material_counter;
			const int32 iCurMatIndex = iOrgNumMaterials+m;
			int32 k=0;
			assert(iCurMatIndex<m_pScene->GetNumMaterials() && iCurMatIndex>=0);
			while(bUnique && k<iCurMatIndex)
				bUnique = m_pScene->GetMaterial(k++)!=m_pScene->GetMaterial(iCurMatIndex);
			if(!bUnique)
			{
				m_pScene->delMaterial(iCurMatIndex);
				--material_counter;
			}
		}
	}

	// colapse redundant meshes due
	// to multiple scene files loaded
	if(mesh_counter > 0)
	{
		for(int m=mesh_counter-1; m>=0; m--)
		{
			bool bUnique = true;
			const int32 iOrgNumMeshes = m_pScene->GetNumMeshes()-mesh_counter;
			const int32 iCurMeshIndex = iOrgNumMeshes+m;
			int32 k=0;
			assert(iCurMeshIndex<m_pScene->GetNumMeshes() && iCurMeshIndex>=0);
			while(bUnique && k<iCurMeshIndex)
				bUnique = m_pScene->GetMesh(k++)!=m_pScene->GetMesh(iCurMeshIndex);
			if(!bUnique)
			{
				m_pScene->delMesh(iCurMeshIndex);
				--mesh_counter;
			}
		}
	}

	// verify names are unique since the removal
	// of redundant meshes and materials
#ifndef NDEBUG
	for(int m=0; m<m_pScene->GetNumMeshes(); m++)
		for(int m2=0; m2<m; m2++)
			assert(strcmp(m_pScene->GetMesh(m).GetName(), m_pScene->GetMesh(m2).GetName()) != 0);

	for(int m=0; m<m_pScene->GetNumMaterials(); m++)
		for(int m2=0; m2<m; m2++)
			assert(strcmp(m_pScene->GetMaterial(m).GetName(), m_pScene->GetMaterial(m2).GetName()) != 0);
#endif

	// equip all mesh instances with a secret
	// pointer to the mother scene
	const int32 iTotInsts = m_pScene->GetNumMeshInstances();
	for(int m=iTotInsts-meshinstance_counter; m<iTotInsts; m++)
		m_pScene->m_pMeshInstances[m].m_pOwnerScene = m_pScene;

	// equip all surfaces with a secret
	// pointer to the mother scene
	const int32 iTotMeshes = m_pScene->GetNumMeshes();
	for(int m=iTotMeshes-mesh_counter; m<iTotMeshes; m++)
		for(int s=0; s<m_pScene->GetMesh(m).GetNumSurfaces(); s++)
			m_pScene->GetMesh(m).GetSurface(s).m_pOwnerScene = m_pScene;

	// Do a reordering placing all Meshes
	// with zero surfaces at the end (NullMesh).
	int32 l, r;
	l=0; r=m_pScene->GetNumMeshes()-1;
	while(l<r)
	{
		while(m_pScene->GetMesh(l).GetNumSurfaces()!=0 && l<r)
			++l;
		while(m_pScene->GetMesh(r).GetNumSurfaces()==0 && r>l)
			--r;

		// swap
		if(l<r)
			m_pScene->SwapMeshes(l, r);
	}

	m_pScene->m_iNumNullMeshes = m_pScene->GetNumMeshes()-r;
	if(m_pScene->GetMesh(m_pScene->GetNumMeshes()-1).GetNumSurfaces()!=0)
		--m_pScene->m_iNumNullMeshes;
		
	// Do a reordering placing at the end all MeshInstances
	// refering to a mesh with zero surfaces (NullMesh).
	l=0, r=m_pScene->GetNumMeshInstances()-1;
	while(l<r)
	{
		while(m_pScene->GetMesh( m_pScene->GetMeshInstance(l).GetMeshIndex() ).GetNumSurfaces()!=0 && l<r)
			++l;
		while(m_pScene->GetMesh( m_pScene->GetMeshInstance(r).GetMeshIndex() ).GetNumSurfaces()==0 && r>l)
			--r;

		// swap
		if(l<r)
			m_pScene->SwapMeshInstances(l, r);
	}

	m_pScene->m_iNumNullMeshInstances = m_pScene->GetNumMeshInstances()-r;
	if(m_pScene->GetMesh( m_pScene->GetMeshInstance(m_pScene->GetNumMeshInstances()-1).GetMeshIndex() ).GetNumSurfaces()!=0)
		--m_pScene->m_iNumNullMeshInstances;

	// verify buffer for debugging
#ifndef NDEBUG
	int32 num_null=0, num_null_insts=0;
	bool bGotNullYet=false, bGotNullInstYet=false;
	for(int32 m=0; m<m_pScene->GetNumMeshes(); m++)
		if(m_pScene->GetMesh(m).GetNumSurfaces()==0) { ++num_null; bGotNullYet=true; }
		else assert(!bGotNullYet);
	for(int32 m=0; m<m_pScene->GetNumMeshInstances(); m++)
		if(m_pScene->GetMesh( m_pScene->GetMeshInstance(m).GetMeshIndex() ).GetNumSurfaces()==0) { ++num_null_insts; bGotNullInstYet=true; }
		else assert(!bGotNullInstYet);
	assert(num_null==m_pScene->GetNumNullMeshes() && num_null_insts==m_pScene->GetNumNullMeshInstances());
#endif

	return true;
}

// set anim time
void CSceneCreatorIntmd::SetAnimTime(const float fAnimTime)
{
	m_fAnimTime = fAnimTime;
}

// Material creators
void CSceneCreatorIntmd::MaterialBegin(const char mat_name[])
{
	printf("material begin \"%s\"\n", mat_name);

	assert(GetCurMode() == SCENE_INTMD);
	PushMode(MATERIAL_INTMD);

	// increase buffer by 1 material
	m_pScene->IncMaterialBuffer(mat_name);
}

void CSceneCreatorIntmd::MaterialEnd()
{
	printf("material end\n");

	assert(GetCurMode() == MATERIAL_INTMD);
	PopMode();
	++material_counter;
}

void CSceneCreatorIntmd::DiffTextureBegin()
{
	assert(GetCurMode() == MATERIAL_INTMD);
	PushMode(DIFFTEX_INTMD);
}

void CSceneCreatorIntmd::DiffTextureEnd()
{
	assert(GetCurMode() == DIFFTEX_INTMD);
	PopMode();
}

void CSceneCreatorIntmd::ReflTextureBegin()
{
	assert(GetCurMode() == MATERIAL_INTMD);
	PushMode(REFLTEX_INTMD);
}

void CSceneCreatorIntmd::ReflTextureEnd()
{
	assert(GetCurMode() == REFLTEX_INTMD);
	PopMode();
}

void CSceneCreatorIntmd::BumpTextureBegin()
{
	assert(GetCurMode() == MATERIAL_INTMD);
	PushMode(BUMPTEX_INTMD);
}

void CSceneCreatorIntmd::BumpTextureEnd()
{
	assert(GetCurMode() == BUMPTEX_INTMD);
	PopMode();
}

void CSceneCreatorIntmd::SpecTextureBegin()
{
	assert(GetCurMode() == MATERIAL_INTMD);
	PushMode(SPECTEX_INTMD);
}

void CSceneCreatorIntmd::SpecTextureEnd()
{
	assert(GetCurMode() == SPECTEX_INTMD);
	PopMode();
}



// Mesh creators
void CSceneCreatorIntmd::MeshBegin(const char mesh_name[])
{
	printf("mesh begin \"%s\"\n", mesh_name);

	assert(GetCurMode() == SCENE_INTMD);
	PushMode(MESH_INTMD);

	// increase buffer by 1 mesh
	m_pScene->IncMeshBuffer(mesh_name);
}

void CSceneCreatorIntmd::MeshEnd()
{
	printf("mesh end\n");

	assert(GetCurMode() == MESH_INTMD);
	PopMode();
	++mesh_counter;
}

void CSceneCreatorIntmd::SurfaceBegin(const int numverts, const int numstrips, const char matref[])
{
	printf("SurfaceBegin: %d %d \"%s\"\n", numverts, numstrips, matref);

	assert(GetCurMode() == MESH_INTMD);
	PushMode(SURFACE_INTMD);

	CMeshIntmd &mesh = m_pScene->GetMesh(m_pScene->GetNumMeshes()-1);
	mesh.IncSurfBuffer(numverts, numstrips, matref);

	// set counters to verify completion
	vert_counter=0;
	normal_counter=0;
	tangent_counter=0;
	binormal_counter=0;
	st_counter=0;
	strip_counter=0;
	pSurf = &mesh.GetSurface(mesh.GetNumSurfaces()-1);
}

void CSceneCreatorIntmd::SurfaceEnd()
{
	printf("SurfaceEnd\n");

	assert(GetCurMode() == SURFACE_INTMD);
	PopMode();
	assert(vert_counter == pSurf->GetArrayLen());
	assert(normal_counter == pSurf->GetArrayLen());
	assert(st_counter == pSurf->GetArrayLen());
	assert(strip_counter == pSurf->GetNumStrips());
}

void CSceneCreatorIntmd::StripBegin(const int len)
{
	assert(GetCurMode() == SURFACE_INTMD);
	PushMode(STRIP_INTMD);
	index_counter = 0;	// set counter to verify completion
	assert(strip_counter < pSurf->GetNumStrips());
	pSurf->m_pStrips[strip_counter].Setup(len);
}

void CSceneCreatorIntmd::StripEnd()
{
	assert(GetCurMode() == STRIP_INTMD);
	PopMode();
	assert(index_counter == pSurf->m_pStrips[strip_counter].GetArrayLen());
	++strip_counter;
}

void CSceneCreatorIntmd::AddVertex(const float x, const float y, const float z)
{
	assert(GetCurMode() == SURFACE_INTMD);
	assert(vert_counter < pSurf->GetArrayLen());
	pSurf->m_pVerts[vert_counter++] = Vec3(x, y, z);
}

void CSceneCreatorIntmd::AddNormal(const float nx, const float ny, const float nz)
{
	assert(GetCurMode() == SURFACE_INTMD);
	assert(normal_counter < pSurf->GetArrayLen());
	pSurf->m_pNormals[normal_counter++] = Vec3(nx, ny, nz);
}

void CSceneCreatorIntmd::AddTangent(const float tx, const float ty, const float tz)
{
	assert(GetCurMode() == SURFACE_INTMD);
	assert(tangent_counter < pSurf->GetArrayLen());
	pSurf->m_pTangents[tangent_counter++] = Vec3(tx, ty, tz);
}

void CSceneCreatorIntmd::AddBinormal(const float bx, const float by, const float bz)
{
	assert(GetCurMode() == SURFACE_INTMD);
	assert(binormal_counter < pSurf->GetArrayLen());
	pSurf->m_pBiNormals[binormal_counter++] = Vec3(bx, by, bz);
}

void CSceneCreatorIntmd::AddTexCoord(const float s, const float t)
{
	assert(GetCurMode() == SURFACE_INTMD);
	assert(st_counter < pSurf->GetArrayLen());
	pSurf->m_pSTs[st_counter++] = Vec3(s, t, 1.0f);
}

void CSceneCreatorIntmd::AddIndex(const int32 index)
{
	assert(GetCurMode() == STRIP_INTMD);
	assert(index_counter < pSurf->m_pStrips[strip_counter].GetArrayLen());
	pSurf->m_pStrips[strip_counter].m_pIndices[index_counter++] = index;
}

// Meshinstance creators
void CSceneCreatorIntmd::MeshinstanceBegin(const char meshinst_name[])
{
	printf("meshinst begin \"%s\"\n", meshinst_name);

	assert(GetCurMode() == SCENE_INTMD);
	PushMode(MESHINSTANCE_INTMD);

	// increase buffer by 1 meshinstance
	m_pScene->IncMeshInstanceBuffer(meshinst_name);
}

void CSceneCreatorIntmd::MeshinstanceEnd()
{
	printf("meshinst end\n");

	assert(GetCurMode() == MESHINSTANCE_INTMD);
	PopMode();
	++meshinstance_counter;
}

void CSceneCreatorIntmd::SetMeshRef(const char meshref[])
{
	printf("meshref \"%s\"\n", meshref);

	assert(GetCurMode() == MESHINSTANCE_INTMD);
	CMeshInstanceIntmd &meshinst = m_pScene->GetMeshInstance(m_pScene->GetNumMeshInstances()-1);
	meshinst.SetMeshref(meshref);
}

// common animation creators
void CSceneCreatorIntmd::PositionBegin(const int32 array_len, const float fDuration)
{
	CKey3ListIntmd * pPosArray = NULL;
	switch(GetCurMode())
	{
		case MESHINSTANCE_INTMD:
			pPosArray = &m_pScene->GetMeshInstance(m_pScene->GetNumMeshInstances()-1).m_vPosArray;
			break;
		case CAMERA_INTMD:
			pPosArray = &m_pScene->GetCamera(m_pScene->GetNumCameras()-1).m_vPosArray;
			break;
		case LIGHT_INTMD:
			pPosArray = &m_pScene->GetLight(m_pScene->GetNumLights()-1).m_vPosArray;
			break;
		default:
			assert(false);
	};
	
	PushMode(POSITION_INTMD);
	assert(pPosArray->m_pKeys == NULL && pPosArray->GetNumKeys()==0);
	pPosArray->m_fDuration = (fDuration>0) ? fDuration : m_fAnimTime;
	pPosArray->m_iNumKeys = array_len;	
	pPosArray->m_pKeys = new Vec3[array_len];
}

void CSceneCreatorIntmd::PositionEnd()
{
	assert(GetCurMode() == POSITION_INTMD);
	PopMode();
}

void CSceneCreatorIntmd::QuaternionBegin(const int32 array_len, const float fDuration)
{
	CQuatListIntmd * pRotArray = NULL;
	switch(GetCurMode())
	{
		case MESHINSTANCE_INTMD:
			pRotArray = &m_pScene->GetMeshInstance(m_pScene->GetNumMeshInstances()-1).m_qRotArray;
			break;
		case CAMERA_INTMD:
			pRotArray = &m_pScene->GetCamera(m_pScene->GetNumCameras()-1).m_qRotArray;
			break;
		case LIGHT_INTMD:
			pRotArray = &m_pScene->GetLight(m_pScene->GetNumLights()-1).m_qRotArray;
			break;
		default:
			assert(false);
	};
	
	PushMode(QUATERNION_INTMD);
	assert(pRotArray->m_pKeys == NULL && pRotArray->GetNumKeys()==0);
	pRotArray->m_fDuration = (fDuration>0) ? fDuration : m_fAnimTime;
	pRotArray->m_iNumKeys = array_len;
	pRotArray->m_pKeys = new Quat[array_len];
}

void CSceneCreatorIntmd::QuaternionEnd()
{
	assert(GetCurMode() == QUATERNION_INTMD);
	PopMode();
}

void CSceneCreatorIntmd::ScaleBegin(const int32 array_len, const float fDuration)
{
	CKey1ListIntmd * pScaleArray = NULL;
	switch(GetCurMode())
	{
		case MESHINSTANCE_INTMD:
			pScaleArray = &m_pScene->GetMeshInstance(m_pScene->GetNumMeshInstances()-1).m_fUniScaleArray;
			break;
		case CAMERA_INTMD:
			pScaleArray = &m_pScene->GetCamera(m_pScene->GetNumCameras()-1).m_fUniScaleArray;
			break;
		case LIGHT_INTMD:
			pScaleArray = &m_pScene->GetLight(m_pScene->GetNumLights()-1).m_fUniScaleArray;
			break;
		default:
			assert(false);
	};

	AllocKey1Array(pScaleArray, SCALE_INTMD, array_len, fDuration);
}

void CSceneCreatorIntmd::ScaleEnd()
{
	assert(GetCurMode() == SCALE_INTMD);
	PopMode();
}	
	
// Camera creators
void CSceneCreatorIntmd::CameraBegin(const char camera_name[])
{
	printf("camera begin \"%s\"\n", camera_name);

	assert(GetCurMode() == SCENE_INTMD);
	PushMode(CAMERA_INTMD);

	// increase buffer by 1 camera
	m_pScene->IncCameraBuffer(camera_name);
}

void CSceneCreatorIntmd::CameraEnd()
{
	printf("camera end\n");

	assert(GetCurMode() == CAMERA_INTMD);
	PopMode();
}

// camera specific animation creators
void CSceneCreatorIntmd::FieldOfViewBegin(const int32 array_len, const float fDuration)
{
	assert(GetCurMode() == CAMERA_INTMD);
	AllocKey1Array(&m_pScene->GetCamera(m_pScene->GetNumCameras()-1).m_fFovArray, FIELDOFVIEW_INTMD, array_len, fDuration);
}

void CSceneCreatorIntmd::FieldOfViewEnd()
{
	assert(GetCurMode() == FIELDOFVIEW_INTMD);
	PopMode();
}

void CSceneCreatorIntmd::SetRatio(const float fAspect)
{
	assert(GetCurMode() == CAMERA_INTMD);
	m_pScene->GetCamera(m_pScene->GetNumCameras()-1).m_fAspect = fAspect;
}

// light creators
void CSceneCreatorIntmd::LightBegin(const char light_name[])
{
	printf("light begin \"%s\"\n", light_name);

	assert(GetCurMode() == SCENE_INTMD);
	PushMode(LIGHT_INTMD);

	// increase buffer by 1 light
	m_pScene->IncLightBuffer(light_name);
}

void CSceneCreatorIntmd::LightEnd()
{
	printf("light end\n");

	assert(GetCurMode() == LIGHT_INTMD);
	PopMode();
}


// light specific animation creators
void CSceneCreatorIntmd::Color3fBegin(const int32 array_len, const float fDuration)
{
	assert(GetCurMode() == LIGHT_INTMD);
	CKey3ListIntmd * pColArray = &m_pScene->GetLight(m_pScene->GetNumLights()-1).m_vColorArray;
	
	PushMode(COLOR3F_INTMD);
	assert(pColArray->m_pKeys == NULL && pColArray->GetNumKeys()==0);
	pColArray->m_fDuration = (fDuration>0) ? fDuration : m_fAnimTime;
	pColArray->m_iNumKeys = array_len;
	pColArray->m_pKeys = new Vec3[array_len];
}

void CSceneCreatorIntmd::Color3fEnd()
{
	assert(GetCurMode() == COLOR3F_INTMD);
	PopMode();
}

void CSceneCreatorIntmd::NearBegin(const int32 array_len, const float fDuration)
{
	assert(GetCurMode() == LIGHT_INTMD);
	AllocKey1Array(&m_pScene->GetLight(m_pScene->GetNumLights()-1).m_fNearArray, NEAR_INTMD, array_len, fDuration);
}

void CSceneCreatorIntmd::NearEnd()
{
	assert(GetCurMode() == NEAR_INTMD);
	PopMode();
}

void CSceneCreatorIntmd::FarBegin(const int32 array_len, const float fDuration)
{
	assert(GetCurMode() == LIGHT_INTMD);
	AllocKey1Array(&m_pScene->GetLight(m_pScene->GetNumLights()-1).m_fFarArray, FAR_INTMD, array_len, fDuration);
}

void CSceneCreatorIntmd::FarEnd()
{
	assert(GetCurMode() == FAR_INTMD);
	PopMode();
}

void CSceneCreatorIntmd::HotSpotBegin(const int32 array_len, const float fDuration)
{
	assert(GetCurMode() == LIGHT_INTMD);
	AllocKey1Array(&m_pScene->GetLight(m_pScene->GetNumLights()-1).m_fHotSpotArray, HOTSPOT_INTMD, array_len, fDuration);
}

void CSceneCreatorIntmd::HotSpotEnd()
{
	assert(GetCurMode() == HOTSPOT_INTMD);
	PopMode();
}

void CSceneCreatorIntmd::FallOffBegin(const int32 array_len, const float fDuration)
{
	assert(GetCurMode() == LIGHT_INTMD);
	AllocKey1Array(&m_pScene->GetLight(m_pScene->GetNumLights()-1).m_fFallOffArray, FALLOFF_INTMD, array_len, fDuration);
}

void CSceneCreatorIntmd::FallOffEnd()
{
	assert(GetCurMode() == FALLOFF_INTMD);
	PopMode();
}

void CSceneCreatorIntmd::AspectBegin(const int32 array_len, const float fDuration)
{
	assert(GetCurMode() == LIGHT_INTMD);
	AllocKey1Array(&m_pScene->GetLight(m_pScene->GetNumLights()-1).m_fAspectArray, ASPECT_INTMD, array_len, fDuration);
}

void CSceneCreatorIntmd::AspectEnd()
{
	assert(GetCurMode() == ASPECT_INTMD);
	PopMode();
}

void CSceneCreatorIntmd::SetSpot()
{
	assert(GetCurMode() == LIGHT_INTMD);
	m_pScene->GetLight(m_pScene->GetNumLights()-1).m_bIsSpot = true;
}

void CSceneCreatorIntmd::SetConeRectangular()
{
	assert(GetCurMode() == LIGHT_INTMD);
	m_pScene->GetLight(m_pScene->GetNumLights()-1).m_bIsRectangular = true;
}

void CSceneCreatorIntmd::SetConeCircular()
{
	assert(GetCurMode() == LIGHT_INTMD);
	m_pScene->GetLight(m_pScene->GetNumLights()-1).m_bIsRectangular = false;
}


void CSceneCreatorIntmd::SetKey1(const int32 index, const float val)
{
	const INTMD_MODE mode = GetCurMode();
	PopMode();	// need to get to parent mode

	switch(mode)
	{
		case SCALE_INTMD:
			if(GetCurMode()==MESHINSTANCE_INTMD)
				m_pScene->GetMeshInstance(m_pScene->GetNumMeshInstances()-1).m_fUniScaleArray.m_pKeys[index] = val;
			else if(GetCurMode()==CAMERA_INTMD)
				m_pScene->GetCamera(m_pScene->GetNumCameras()-1).m_fUniScaleArray.m_pKeys[index] = val;
			else if(GetCurMode()==LIGHT_INTMD)
				m_pScene->GetLight(m_pScene->GetNumLights()-1).m_fUniScaleArray.m_pKeys[index] = val;
			else
				assert(false);
			break;
		case FIELDOFVIEW_INTMD:
				assert(GetCurMode()==CAMERA_INTMD);
				m_pScene->GetCamera(m_pScene->GetNumCameras()-1).m_fFovArray.m_pKeys[index] = val;
			break;
		case NEAR_INTMD:
				assert(GetCurMode()==LIGHT_INTMD);
				m_pScene->GetLight(m_pScene->GetNumLights()-1).m_fNearArray.m_pKeys[index] = val;
			break;
		case FAR_INTMD:
				assert(GetCurMode()==LIGHT_INTMD);
				m_pScene->GetLight(m_pScene->GetNumLights()-1).m_fFarArray.m_pKeys[index] = val;
			break;
		case HOTSPOT_INTMD:
				assert(GetCurMode()==LIGHT_INTMD);
				m_pScene->GetLight(m_pScene->GetNumLights()-1).m_fHotSpotArray.m_pKeys[index] = val;
			break;
		case FALLOFF_INTMD:
				assert(GetCurMode()==LIGHT_INTMD);
				m_pScene->GetLight(m_pScene->GetNumLights()-1).m_fFallOffArray.m_pKeys[index] = val;
			break;
		case ASPECT_INTMD:
				assert(GetCurMode()==LIGHT_INTMD);
				m_pScene->GetLight(m_pScene->GetNumLights()-1).m_fAspectArray.m_pKeys[index] = val;
			break;
		default:
			assert(false);
	}

	// restore mode and finish
	PushMode(mode);
}

void CSceneCreatorIntmd::SetKey3(const int32 index, const float x, const float y, const float z)
{
	const INTMD_MODE mode = GetCurMode();
	PopMode();	// need to get to parent mode

	switch(GetCurMode())
	{
		case MESHINSTANCE_INTMD:
			assert(mode==POSITION_INTMD);
			m_pScene->GetMeshInstance(m_pScene->GetNumMeshInstances()-1).m_vPosArray.m_pKeys[index] = Vec3(x,y,z);
			break;
		case CAMERA_INTMD:
			assert(mode==POSITION_INTMD);
			m_pScene->GetCamera(m_pScene->GetNumCameras()-1).m_vPosArray.m_pKeys[index] = Vec3(x,y,z);
			break;
		case LIGHT_INTMD:
			assert(mode==POSITION_INTMD || mode==COLOR3F_INTMD);
			if(mode==POSITION_INTMD)
				m_pScene->GetLight(m_pScene->GetNumLights()-1).m_vPosArray.m_pKeys[index] = Vec3(x,y,z);
			else
				m_pScene->GetLight(m_pScene->GetNumLights()-1).m_vColorArray.m_pKeys[index] = Vec3(x,y,z);
			break;
		default:
			assert(false);
	}

	// restore mode and finish
	PushMode(mode);
}

void CSceneCreatorIntmd::SetQuat(const int32 index, const float x, const float y, const float z, const float w)
{
	assert(GetCurMode() == QUATERNION_INTMD);
	PopMode();	// need to get to parent mode

	switch(GetCurMode())
	{
		case MESHINSTANCE_INTMD:
			m_pScene->GetMeshInstance(m_pScene->GetNumMeshInstances()-1).m_qRotArray.m_pKeys[index] = Quat(Vec3(x,y,z), w);
			break;
		case CAMERA_INTMD:
			m_pScene->GetCamera(m_pScene->GetNumCameras()-1).m_qRotArray.m_pKeys[index] = Quat(Vec3(x,y,z), w);
			break;
		case LIGHT_INTMD:
			m_pScene->GetLight(m_pScene->GetNumLights()-1).m_qRotArray.m_pKeys[index] = Quat(Vec3(x,y,z), w);
			break;
		default:
			assert(false);
	}

	// restore mode and finish
	PushMode(QUATERNION_INTMD);
}

// helper anim function
void CSceneCreatorIntmd::AllocKey1Array(CKey1ListIntmd * pKey1Array, const INTMD_MODE mode, const int32 array_len, const float fDuration)
{	
	PushMode(mode);
	assert(pKey1Array->m_pKeys == NULL && pKey1Array->GetNumKeys()==0);
	pKey1Array->m_fDuration = (fDuration>0) ? fDuration : m_fAnimTime;
	pKey1Array->m_iNumKeys = array_len;
	pKey1Array->m_pKeys = new float[array_len];
	assert(pKey1Array->m_pKeys != NULL);
}



// Texture setup handling functions
bool CSceneCreatorIntmd::TextureBegin(const uint32 uChkID)
{
	// return true if this was already
	// a registered texture.
	PushMode(TEXTURE_INTMD);
	
	m_bAlreadyRegisteredTex=false;
	int t=0;
	while(t<m_pScene->GetNumTextures() && !m_bAlreadyRegisteredTex)
	{
		if(m_pScene->GetTexture(t).m_uChkID != uChkID)	
			++t;
		else
			m_bAlreadyRegisteredTex=true;
	}

	// texture already present
	if(m_bAlreadyRegisteredTex)
		m_pTexBuild = &m_pScene->m_pTextures[t];
	else
	{
		// alloc tmp texture
		assert(m_pTexBuild == NULL);
		m_pTexBuild = new CTextureIntmd;
		assert(m_pTexBuild != NULL);

		// set counter
		mipmap_counter = 0;
		m_pTexBuild->m_uChkID = uChkID;
	}

	return m_bAlreadyRegisteredTex;
}

void CSceneCreatorIntmd::TextureEnd()
{
	assert(GetCurMode() == TEXTURE_INTMD);
	PopMode();

	// if this was a new chunkID
	if(!m_bAlreadyRegisteredTex)
	{
		// backup pointer
		CTextureIntmd * pOldTex = m_pTexBuild;
		
		// insert m_pTexBuild into list
		bool bFoundMatch;
		m_pTexBuild = m_pScene->InsertIntoTexList( m_pTexBuild, &bFoundMatch );

		// cleanup
		if(bFoundMatch)
		{
			for(int f=0; f<pOldTex->GetNumLevels(); f++)
			{
				assert(pOldTex->m_pMipMaps[f].m_pTexelData != NULL);
				delete [] pOldTex->m_pMipMaps[f].m_pTexelData;
			}
			assert(pOldTex->m_pMipMaps != NULL);
			assert(pOldTex->m_pUserDefProps != NULL);
			delete [] pOldTex->m_pMipMaps;
			delete [] pOldTex->m_pUserDefProps;
			if(pOldTex->m_pInternPalette != NULL)
				delete [] pOldTex->m_pInternPalette;
		}
		delete pOldTex; pOldTex = NULL;
	}
}

void CSceneCreatorIntmd::MipmapBegin()
{
	assert(!m_bAlreadyRegisteredTex);
	assert(GetCurMode() == TEXTURE_INTMD);
	PushMode(MIPMAP_INTMD);
}

void CSceneCreatorIntmd::MipmapEnd()
{
	assert(!m_bAlreadyRegisteredTex);
	assert(GetCurMode() == MIPMAP_INTMD);
	PopMode();

	// inc counter
	++mipmap_counter;
}

void CSceneCreatorIntmd::SetTexDuration(const float fDuration)
{
	switch(GetCurMode())
	{
		case DIFFTEX_INTMD:
			m_pScene->m_pMaterials[m_pScene->GetNumMaterials()-1].m_cDiffTexture.m_fDuration = fDuration;
			break;
		case REFLTEX_INTMD:
			m_pScene->m_pMaterials[m_pScene->GetNumMaterials()-1].m_cReflTexture.m_fDuration = fDuration;
			break;
		case BUMPTEX_INTMD:
			m_pScene->m_pMaterials[m_pScene->GetNumMaterials()-1].m_cBumpTexture.m_fDuration = fDuration;
			break;
		case SPECTEX_INTMD:
			m_pScene->m_pMaterials[m_pScene->GetNumMaterials()-1].m_cSpecTexture.m_fDuration = fDuration;
			break;
		default:
			assert(false);
	}
}

void CSceneCreatorIntmd::AddLatestTextureToSeq()
{
	const int32 tex_frame = m_pTexBuild - m_pScene->m_pTextures;
		
	switch(GetCurMode())
	{
		case DIFFTEX_INTMD:
			m_pScene->m_pMaterials[m_pScene->GetNumMaterials()-1].m_cDiffTexture.IncTexBuffer(tex_frame);
			break;
		case REFLTEX_INTMD:
			m_pScene->m_pMaterials[m_pScene->GetNumMaterials()-1].m_cReflTexture.IncTexBuffer(tex_frame);
			break;
		case BUMPTEX_INTMD:
			m_pScene->m_pMaterials[m_pScene->GetNumMaterials()-1].m_cBumpTexture.IncTexBuffer(tex_frame);
			break;
		case SPECTEX_INTMD:
			m_pScene->m_pMaterials[m_pScene->GetNumMaterials()-1].m_cSpecTexture.IncTexBuffer(tex_frame);
			break;
		default:
			assert(false);
	}

	// mark this as cleared
	m_pTexBuild = NULL;
}

void CSceneCreatorIntmd::SetUserDefProps(const char userdefprops[])
{
	// verify
	assert(!m_bAlreadyRegisteredTex);
	assert(GetCurMode() == TEXTURE_INTMD);
	assert(m_pTexBuild->m_pUserDefProps == NULL);

	// make a copy
	m_pTexBuild->m_pUserDefProps = new char[strlen(userdefprops)+1];
	assert(m_pTexBuild->m_pUserDefProps != NULL);
	strcpy(m_pTexBuild->m_pUserDefProps, userdefprops);
}

void CSceneCreatorIntmd::SetPixFormat(const int32 pix_format)
{
	// verify
	assert(!m_bAlreadyRegisteredTex);
	assert(GetCurMode() == TEXTURE_INTMD);
	assert(m_pTexBuild->m_pInternPalette==NULL);

	// set value
	m_pTexBuild->m_iPixelFormat = pix_format;

	// alloc palette if required
	if(pix_format==INDEXCOLOR16 || pix_format==INDEXCOLOR256)
	{
		const int32 num_cols = (pix_format==INDEXCOLOR16) ? 16 : 256;
		m_pTexBuild->m_pInternPalette = new uint8[num_cols*4];
	}
	else
		assert(pix_format==COLORRGBA32);
}

void CSceneCreatorIntmd::SetColor(const int32 index, const uint8 r, const uint8 g, const uint8 b, const uint8 a)
{
	// verify
	assert(!m_bAlreadyRegisteredTex);
	assert(GetCurMode() == TEXTURE_INTMD);
	assert(m_pTexBuild->m_pInternPalette!=NULL);
	
	if(m_pTexBuild->m_iPixelFormat==INDEXCOLOR16 || m_pTexBuild->m_iPixelFormat==INDEXCOLOR256)
	{
		const int32 num_cols = (m_pTexBuild->m_iPixelFormat==INDEXCOLOR16) ? 16 : 256;
		assert(index < num_cols);
		uint8 * palPtr = (uint8 *) m_pTexBuild->m_pInternPalette;
		palPtr[index*4+0]=r;
		palPtr[index*4+1]=g;
		palPtr[index*4+2]=b;
		palPtr[index*4+3]=a;
	}
	else
		assert(false);
}

void CSceneCreatorIntmd::SetNumLevels(const int32 num_levels)
{
	// verify
	assert(!m_bAlreadyRegisteredTex);
	assert(GetCurMode() == TEXTURE_INTMD);
	assert(m_pTexBuild->m_pMipMaps == NULL);

	// alloc levels
	m_pTexBuild->m_pMipMaps = new CBinaryBitmapIntmd[num_levels];
	assert(m_pTexBuild->m_pMipMaps != NULL);
	m_pTexBuild->m_iNumLevels = num_levels;
}

void CSceneCreatorIntmd::SetDimension(const int32 width, const int32 height)
{
	// verify
	assert(!m_bAlreadyRegisteredTex);
	assert(GetCurMode() == MIPMAP_INTMD);
	assert(m_pTexBuild->m_pMipMaps[mipmap_counter].m_pTexelData == NULL);

	// set values
	const int32 pix_size = (m_pTexBuild->m_iPixelFormat==COLORRGBA32) ? 4 : 1; 
	
	CBinaryBitmapIntmd * bmap = &m_pTexBuild->m_pMipMaps[mipmap_counter];
	bmap->m_iWidth = width;
	bmap->m_iHeight = height;
	bmap->m_pTexelData = new uint8[width*height*pix_size];
	assert(bmap->m_pTexelData!=NULL);
}

void CSceneCreatorIntmd::SetTexel(const int32 x, const int32 y, const uint8 pen)
{
	// verify
	assert(!m_bAlreadyRegisteredTex);
	assert(GetCurMode() == MIPMAP_INTMD);
	assert(m_pTexBuild->m_iPixelFormat==INDEXCOLOR16 || m_pTexBuild->m_iPixelFormat==INDEXCOLOR256);
	assert(m_pTexBuild->m_pMipMaps[mipmap_counter].m_pTexelData != NULL);

	// set pen
	((uint8 *) m_pTexBuild->m_pMipMaps[mipmap_counter].m_pTexelData)[y*m_pTexBuild->m_pMipMaps[mipmap_counter].m_iWidth + x]=pen;
}

void CSceneCreatorIntmd::SetTexel(const int32 x, const int32 y, const uint8 r, const uint8 g, const uint8 b, const uint8 a)
{
	// verify
	assert(!m_bAlreadyRegisteredTex);
	assert(GetCurMode() == MIPMAP_INTMD);
	assert(m_pTexBuild->m_iPixelFormat==COLORRGBA32);
	assert(m_pTexBuild->m_pMipMaps[mipmap_counter].m_pTexelData != NULL);

	// set color
	((uint8 *) m_pTexBuild->m_pMipMaps[mipmap_counter].m_pTexelData)[(y*m_pTexBuild->m_pMipMaps[mipmap_counter].m_iWidth + x)*4+0]=r;
	((uint8 *) m_pTexBuild->m_pMipMaps[mipmap_counter].m_pTexelData)[(y*m_pTexBuild->m_pMipMaps[mipmap_counter].m_iWidth + x)*4+1]=g;
	((uint8 *) m_pTexBuild->m_pMipMaps[mipmap_counter].m_pTexelData)[(y*m_pTexBuild->m_pMipMaps[mipmap_counter].m_iWidth + x)*4+2]=b;
	((uint8 *) m_pTexBuild->m_pMipMaps[mipmap_counter].m_pTexelData)[(y*m_pTexBuild->m_pMipMaps[mipmap_counter].m_iWidth + x)*4+3]=a;
}



// constructor & destructor
void CSceneCreatorIntmd::CleanUp()
{
	if(m_pScene!=NULL)
	{
		m_pScene->CleanUp();
		delete m_pScene; m_pScene=NULL;
	}

	m_iCurDepth = 0;	// init stack
	m_eModeStack[0] = BLANK_INTMD;
	m_pTexBuild = NULL;	// used for setup of new textures
	m_bAlreadyRegisteredTex=false;
	m_fAnimTime=-1;
}

void CSceneCreatorIntmd::CleanUpMeshes()
{
	if(m_pScene!=NULL)
		m_pScene->CleanUpMeshes();
}

void CSceneCreatorIntmd::CleanUpTextures()
{
	if(m_pScene!=NULL)
		m_pScene->CleanUpTextures();
	m_pTexBuild = NULL;	// used for setup of new textures
	m_bAlreadyRegisteredTex=false;
}

CSceneCreatorIntmd::CSceneCreatorIntmd()
{
	m_pScene = NULL;
	m_iCurDepth = 0;	// init stack
	m_eModeStack[0] = BLANK_INTMD;
	m_pTexBuild = NULL;	// used for setup of new textures
	m_bAlreadyRegisteredTex=false;
	m_fAnimTime=-1;
}

CSceneCreatorIntmd::~CSceneCreatorIntmd()
{


}
