#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "p2lassert.h"
#include "intmd/scenecreatorparse.h"
#include "intmd/pixel_formats.h"
#include <geommath/geommath.h>


bool CSceneCreatorParse::LoadToScene(IFileHandler * pFileSc, IFileHandler * pFileGeo)
{
	// fetch files
	pFileScene = pFileSc;
	pFileGeomData = pFileGeo;
	
	// reset
	pFileGeomData->LocateStart();
	pFileScene->LocateStart();

	//
	szeScene = pFileScene->Size(); 
	pScene = NULL;
	if((pScene = new char[szeScene]) == NULL) assert(false);
	pFileScene->Read(pScene, szeScene);

	// Token parsing
	token.cptr = pScene;
	token.str_len = 0;
	token.token_type = UNKNOWN;

	const bool bSuccess = ParseScene();
	delete [] pScene; pScene = NULL;
	pFileScene = NULL;
	pFileGeomData = NULL;

	return bSuccess;
}

// constructor & destructor
CSceneCreatorParse::CSceneCreatorParse()
{
}

CSceneCreatorParse::~CSceneCreatorParse()
{
}

/////////////////////////////////////////////////////////////////////////////////
////////////////////////////// token parsing ////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

// return parse success bool
bool CSceneCreatorParse::ParseScene()
{
	bool bTokensLeft = true;
	SceneBegin();

	while(bTokensLeft)
	{
		// get us a valid Token
		while(bTokensLeft && token.token_type == UNKNOWN)
			bTokensLeft = match(token.token_type);

		switch(token.token_type)
		{
			case ANIMTIME:
				bTokensLeft = ParseAnimTime();
				break;
			case MATERIAL:
				bTokensLeft = ParseMaterial();
				break;
			case MESH:
				bTokensLeft = ParseMesh();
				break;
			case MESHINSTANCE:
				bTokensLeft = ParseMeshInstance();
				break;
			case CAMERA:
				bTokensLeft = ParseCamera();
				break;
			case LIGHT:
				bTokensLeft = ParseLight();
				break;
			default:
				assert(false);
		}
	}

	return SceneEnd();
}


// return end of buffer bool
bool CSceneCreatorParse::ParseAnimTime()
{
	match(ANIMTIME);
	match(LESS_THEN);
	assert(token.token_type==INTEGER || token.token_type==REAL_VALUE);
	SetAnimTime( token_to_real() );
	match(token.token_type);

	return match(GREATER_THEN);
}

// return end of buffer bool
bool CSceneCreatorParse::ParseMaterial()
{
	match(MATERIAL);
	match(LESS_THEN);

	// fetch name of material
	assert(token.token_type == STRING);
	char * material_name = NULL;
	if((material_name = new char[token.str_len+1]) == NULL) assert(false);
	memcpy(material_name, token.cptr, token.str_len);
	material_name[token.str_len] = '\0';

	// init material
	MaterialBegin(material_name);

	match(STRING);
	match(GREATER_THEN);
	match(LEFT_CURLY);

	// parse token string
	while(token.token_type!=RIGHT_CURLY)
	{
		// pick up material data
		switch(token.token_type)
		{
			case DIFFUSE:
			{
				DiffTextureBegin();
				ParseTexSeq(DIFFUSE);
				DiffTextureEnd();
			}
			break;
			case REFLECTION:
			{
				ReflTextureBegin();
				ParseTexSeq(REFLECTION);
				ReflTextureEnd();
			}
			break;
			case BUMPMAP:
			{
				BumpTextureBegin();
				ParseTexSeq(BUMPMAP);
				BumpTextureEnd();
			}
			break;
			case SPECULAR:
			{
				SpecTextureBegin();
				ParseTexSeq(SPECULAR);
				SpecTextureEnd();
			}
			break;
			default:
				assert(false);
		}
	}

	// end material
	MaterialEnd();
	delete [] material_name; material_name = NULL;

	// returning end of buffer bool
	return match(RIGHT_CURLY);
}

// return end of buffer bool
bool CSceneCreatorParse::ParseMesh()
{
	match(MESH);
	match(LESS_THEN);

	// fetch name of mesh (ID)
	assert(token.token_type == STRING);
	char * mesh_name = NULL;
	if((mesh_name = new char[token.str_len+1]) == NULL) assert(false);
	memcpy(mesh_name, token.cptr, token.str_len);
	mesh_name[token.str_len] = '\0';

	// init mesh
	MeshBegin(mesh_name);

	match(STRING);
	match(GREATER_THEN);
	match(LEFT_CURLY);

	// parse token string
	bool bTokensLeft = true;
	while(bTokensLeft && token.token_type!=RIGHT_CURLY)
	{
		// pick up mesh data
		switch(token.token_type)
		{
			case SURFACE:
			{
				match(SURFACE);
				match(LESS_THEN);
				assert(token.token_type==STRING && token.str_len==4);
				const char cID[]={token.cptr[0], token.cptr[1], token.cptr[2], token.cptr[3], '\0'};
				match(STRING);
				match(COMMA);
				assert(token.token_type==STRING);
				char * matref = NULL;
				if((matref = new char[token.str_len+1]) == NULL) assert(false);
				memcpy(matref, token.cptr, token.str_len);
				matref[token.str_len] = '\0';
				match(STRING);
				match(GREATER_THEN);
				uint32 uBSize = LocateID(cID);	// lookup surface in geomdata
				int32 iGetCurPos = pFileGeomData->GetCurPos();
				int32 numverts, numstrips;
				pFileGeomData->Read(&numverts, sizeof(int32));
				pFileGeomData->Read(&numstrips, sizeof(int32));

				SurfaceBegin(numverts, numstrips, matref);
					for(int v=0; v<numverts; v++)
					{
						float vert[3];
						pFileGeomData->Read(vert, sizeof(float)*3);
						AddVertex(vert[0], vert[1], vert[2]);
					}
					for(int n=0; n<numverts; n++)
					{
						float norm[3];
						pFileGeomData->Read(norm, sizeof(float)*3);
						AddNormal(norm[0], norm[1], norm[2]);
					}
					for(int t=0; t<numverts; t++)
					{
						float tang[3];
						pFileGeomData->Read(tang, sizeof(float)*3);
						AddTangent(tang[0], tang[1], tang[2]);
					}
					for(int b=0; b<numverts; b++)
					{
						float bino[3];
						pFileGeomData->Read(bino, sizeof(float)*3);
						AddBinormal(bino[0], bino[1], bino[2]);
					}
					for(int st=0; st<numverts; st++)
					{
						float mapc[2];
						pFileGeomData->Read(mapc, sizeof(float)*2);
						AddTexCoord(mapc[0], mapc[1]);
					}
					for(int s=0; s<numstrips; s++)
					{
						int32 len;
						pFileGeomData->Read(&len, sizeof(int32));
						StripBegin(len);
							for(int i=0; i<len; i++)
							{
								int32 index;
								pFileGeomData->Read(&index, sizeof(int32));
								assert(index < numverts);
								AddIndex(index);
							}
						StripEnd();
					}
				SurfaceEnd();

				delete [] matref; matref = NULL;
				assert((iGetCurPos+(int) uBSize) == pFileGeomData->GetCurPos());	// verify current location from chunk size
			}
			break;
			default:
				assert(false);
		}
	}

	// end mesh
	MeshEnd();
	delete [] mesh_name; mesh_name = NULL;

	// returning end of buffer bool
	return match(RIGHT_CURLY);
}

// return end of buffer bool
bool CSceneCreatorParse::ParseMeshInstance()
{
	match(MESHINSTANCE);
	match(LESS_THEN);

	// fetch name of meshinstance
	assert(token.token_type == STRING);
	char * meshinstance_name = NULL;
	if((meshinstance_name = new char[token.str_len+1]) == NULL) assert(false);
	memcpy(meshinstance_name, token.cptr, token.str_len);
	meshinstance_name[token.str_len] = '\0';

	// init meshinstance
	MeshinstanceBegin(meshinstance_name);

	match(STRING);
	match(GREATER_THEN);
	match(LEFT_CURLY);

	// parse token string
	bool bTokensLeft = true;
	while(bTokensLeft && token.token_type!=RIGHT_CURLY)
	{
		// pick up meshinstance data
		switch(token.token_type)
		{
			case MESHREF:
			{
				match(MESHREF);
				match(LESS_THEN);

				// fetch name of meshinstance
				assert(token.token_type == STRING);
				char * meshref = NULL;
				if((meshref = new char[token.str_len+1]) == NULL) assert(false);
				memcpy(meshref, token.cptr, token.str_len);
				meshref[token.str_len] = '\0';

				SetMeshRef(meshref);

				delete [] meshref; meshref = NULL;
				match(STRING);
				match(GREATER_THEN);
			}
			break;
			case POSITION:
			case POSITIONARRAY:
				ParseKeys3(token.token_type);
			break;
			case QUATERNION:
			case QUATERNIONARRAY:
				ParseQuats(token.token_type);
				break;
			case SCALE:
			case SCALEARRAY:
				ParseKeys1(token.token_type);
				break;
			default:
				assert(false);
		}
	}

	// end meshinst
	MeshinstanceEnd();
	delete [] meshinstance_name; meshinstance_name = NULL;

	// returning end of buffer bool
	return match(RIGHT_CURLY);
}

//
bool CSceneCreatorParse::ParseTexSeq(const ASC_TOKEN type)
{
	match(type);
	match(LESS_THEN);	

	// parse token string
	bool bTokensLeft = true;
	while(bTokensLeft && token.token_type==STRING)
	{
		// proceed picking up textures in sequence.
		assert(token.token_type==STRING && token.str_len==4);
		const char cID[]={token.cptr[0], token.cptr[1], token.cptr[2], token.cptr[3], '\0'};
		match(STRING);

		// pickup texture
		ParseTexture(cID);

		// add to current texture sequence.
		AddLatestTextureToSeq();

		// next
		if(token.token_type==COMMA)
			bTokensLeft = match(COMMA);
		else
			assert(token.token_type!=STRING);
	}

	// pickup duration
	float fDuration = -1;
	if(token.token_type==INTEGER || token.token_type==REAL_VALUE)
	{
		fDuration = token_to_real();
		match(token.token_type);
	}

	// set duration
	SetTexDuration(fDuration);

	assert(token.token_type==GREATER_THEN);
	return match(GREATER_THEN);
}

bool CSceneCreatorParse::ParseTexture(const char cID[])	// not a 0 terminated string!
{
	const uint32 uChkID = (((uint32) ((uint8) cID[0]))<<0) | (((uint32) ((uint8) cID[1]))<<8) |
								 (((uint32) ((uint8) cID[2]))<<16) | (((uint32) ((uint8) cID[3]))<<24);

	bool bAlreadyRegistered = TextureBegin(uChkID);
			
	if(!bAlreadyRegistered)
	{
		// go to location
		const uint32 uBSize = LocateID(cID);	// lookup surface in geomdata
		const int32 iGetCurPos = pFileGeomData->GetCurPos();

		// determine length of userdefprops
		int32 str_len = 0;
		char ch;
		do
		{
			pFileGeomData->Read(&ch, 1);
			if(ch != '\0')
				++str_len;
		}
		while(ch != '\0');
		pFileGeomData->OffsetFromCur(-(str_len+1));

		// read userdefprops
		char * pUserDefProp = NULL;		
		if((pUserDefProp = new char[str_len+1]) == NULL) assert(false);
		pFileGeomData->Read(pUserDefProp, str_len+1);
		SetUserDefProps( pUserDefProp );

		// Read pixel format
		int32 pix_format;
		pFileGeomData->Read(&pix_format, sizeof(int32));
		SetPixFormat(pix_format);

		// Read palette if one exists
		if(pix_format==INDEXCOLOR16 || pix_format==INDEXCOLOR256)
		{
			const int32 pal_len = (pix_format==INDEXCOLOR16) ? 16 : 256;
			for(int32 c=0; c<pal_len; c++)
			{
				uint8 r,g,b,a;
				pFileGeomData->Read(&r, 1);
				pFileGeomData->Read(&g, 1);
				pFileGeomData->Read(&b, 1);
				pFileGeomData->Read(&a, 1);
				SetColor(c, r, g, b, a);
			}
		}
		else
			assert(pix_format == COLORRGBA32);

		// Read number of mipmap levels
		int32 num_levels;
		pFileGeomData->Read(&num_levels, sizeof(int32));
		SetNumLevels(num_levels);

		// Read mipmaps
		for(int l=0; l<num_levels; l++)
		{
			MipmapBegin();
		
			int32 width, height;
			pFileGeomData->Read(&width, sizeof(int32));
			pFileGeomData->Read(&height, sizeof(int32));
			SetDimension(width, height);

			if(pix_format==INDEXCOLOR16 || pix_format==INDEXCOLOR256)
			{
				for(int y=0; y<height; y++)
					for(int x=0; x<width; x++)
					{
						uint8 pixel;
						pFileGeomData->Read(&pixel, sizeof(uint8));
						SetTexel(x, y, pixel);
					}
			}
			else
			{
				assert(pix_format == COLORRGBA32);
	
				for(int y=0; y<height; y++)
					for(int x=0; x<width; x++)
					{
						uint8 r,g,b,a;
						pFileGeomData->Read(&r, sizeof(uint8));
						pFileGeomData->Read(&g, sizeof(uint8));
						pFileGeomData->Read(&b, sizeof(uint8));
						pFileGeomData->Read(&a, sizeof(uint8));
						SetTexel(x, y, r, g, b, a);
					}
			}
		
			MipmapEnd();	
		}

		// verify location
		assert((iGetCurPos+(int) uBSize) == pFileGeomData->GetCurPos());	// verify current location from chunk size

		// cleanup and return
		delete [] pUserDefProp; pUserDefProp = NULL;
	}
	TextureEnd();

	return true;
}

// return end of buffer bool
bool CSceneCreatorParse::ParseCamera()
{
	match(CAMERA);
	match(LESS_THEN);

	// fetch name of material
	assert(token.token_type == STRING);
	char * camera_name = NULL;
	if((camera_name = new char[token.str_len+1]) == NULL) assert(false);
	memcpy(camera_name, token.cptr, token.str_len);
	camera_name[token.str_len] = '\0';

	// init camera
	CameraBegin(camera_name);

	match(STRING);
	match(GREATER_THEN);
	match(LEFT_CURLY);

	// parse token string
	bool bTokensLeft = true;
	while(bTokensLeft && token.token_type!=RIGHT_CURLY)
	{
		// pick up camera data
		switch(token.token_type)
		{
			case POSITION:
			case POSITIONARRAY:
				ParseKeys3(token.token_type);
				break;
			case QUATERNION:
			case QUATERNIONARRAY:
				ParseQuats(token.token_type);
				break;
			case SCALE:
			case SCALEARRAY:
				ParseKeys1(token.token_type);
				break;
			case FIELDOFVIEW:
			case FIELDOFVIEWARRAY:
				ParseKeys1(token.token_type);
				break;
			case RATIO:
			{
				match(RATIO);
				match(LESS_THEN);
				match(LEFT_BRACKET);
				assert(token.token_type == INTEGER);
				const int32 w = token_to_integer();
				match(INTEGER);
				match(COLON);
				assert(token.token_type == INTEGER);
				const int32 h = token_to_integer();
				match(INTEGER);
				match(RIGHT_BRACKET);
				match(GREATER_THEN);

				SetRatio(((float) w)/h);
			}
			break;
			default:
				assert(false);
		}
	}

	// end camera
	CameraEnd();
	delete [] camera_name; camera_name = NULL;

	// returning end of buffer bool
	return match(RIGHT_CURLY);
}


// return end of buffer bool
bool CSceneCreatorParse::ParseLight()
{
	match(LIGHT);
	match(LESS_THEN);

	// fetch name of material
	assert(token.token_type == STRING);
	char * light_name = NULL;
	if((light_name = new char[token.str_len+1]) == NULL) assert(false);
	memcpy(light_name, token.cptr, token.str_len);
	light_name[token.str_len] = '\0';

	// init light
	LightBegin(light_name);

	match(STRING);
	match(GREATER_THEN);
	match(LEFT_CURLY);

	// parse token string
	bool bTokensLeft = true;
	while(bTokensLeft && token.token_type!=RIGHT_CURLY)
	{
		// pick up light data
		switch(token.token_type)
		{
			case POSITION:
			case POSITIONARRAY:
				ParseKeys3(token.token_type);
				break;
			case QUATERNION:
			case QUATERNIONARRAY:
				ParseQuats(token.token_type);
				break;
			case SCALE:
			case SCALEARRAY:
				ParseKeys1(token.token_type);
				break;
			case COLOR:
			case COLORARRAY:
				ParseKeys3(token.token_type);
				break;
			case NEAR:
			case NEARARRAY:
				ParseKeys1(token.token_type);
				break;
			case FAR:
			case FARARRAY:
				ParseKeys1(token.token_type);
				break;
			case HOTSPOT:
			case HOTSPOTARRAY:
				ParseKeys1(token.token_type);
				break;
			case FALLOFF:
			case FALLOFFARRAY:
				ParseKeys1(token.token_type);
				break;
			case ASPECT:
			case ASPECTARRAY:
				ParseKeys1(token.token_type);
				break;
			case SPOTSHAPE:
				match(SPOTSHAPE);
				match(LESS_THEN);
				assert(token.token_type==CIRCULAR || token.token_type==RECTANGULAR);
				if(token.token_type==CIRCULAR)
				{
					SetConeCircular();
					match(CIRCULAR);
				}
				else
				{
					SetConeRectangular();
					match(RECTANGULAR);
				}
				match(GREATER_THEN);
				break;
			default:
				assert(false);
		}
	}

	// end light
	LightEnd();
	delete [] light_name; light_name = NULL;

	// returning end of buffer bool
	return match(RIGHT_CURLY);
}


// some common component parse functions
bool CSceneCreatorParse::ParseKeys3(const ASC_TOKEN type)
{
	match(type);
	match(LESS_THEN);
	assert(token.token_type==REAL_VALUE || token.token_type==STRING);
	
	float fDuration = -1;
	int32 num_keys, num_channels;
	float x, y, z;
	bool bKeysInBinChunk;
	int32 iGetCurPos=-1;
	uint32 uBSize=0;

	// array length longer then 1
	if(token.token_type==STRING)
	{
		// pickup chunk ID
		assert(token.str_len==4);
		const char cID[]={token.cptr[0], token.cptr[1], token.cptr[2], token.cptr[3], '\0'};
		match(STRING);
		bKeysInBinChunk = true;

		// pickup duration if one is found
		if(token.token_type==COMMA)
		{
			match(COMMA);
			assert(token.token_type == REAL_VALUE);
			fDuration = token_to_real();
			match(REAL_VALUE);
		}

		// locate chunk in binary file
		uBSize = LocateID(cID);	// lookup surface in geomdata
		iGetCurPos = pFileGeomData->GetCurPos();	

		// read num_keys and num_channels
		pFileGeomData->Read(&num_keys, sizeof(int32));
		pFileGeomData->Read(&num_channels, sizeof(int32));
		assert(num_channels == 3);
	}
	else	// otherwise pickup single key in file
	{
		bKeysInBinChunk = false;
		assert(token.token_type == REAL_VALUE);
		x = token_to_real();
		match(REAL_VALUE);
		match(COMMA);

		assert(token.token_type == REAL_VALUE);
		y = token_to_real();
		match(REAL_VALUE);
		match(COMMA);

		assert(token.token_type == REAL_VALUE);
		z = token_to_real();
		match(REAL_VALUE);	
		
		num_keys = 1;
		num_channels = 3;
	}
	match(GREATER_THEN);


	// deliver keys
	switch(type)
	{
		case POSITION:	
		case POSITIONARRAY:
			assert((bKeysInBinChunk && type==POSITIONARRAY) || (!bKeysInBinChunk && type==POSITION));
			PositionBegin(num_keys, fDuration);
			if(bKeysInBinChunk)
				for(int k=0; k<num_keys; k++)
				{
					pFileGeomData->Read(&x, sizeof(float));
					pFileGeomData->Read(&y, sizeof(float));
					pFileGeomData->Read(&z, sizeof(float));
					SetKey3(k, x, y, z);
				}
			else
				SetKey3(0, x, y, z);
			PositionEnd();
			break;
		case COLOR:
		case COLORARRAY:
			assert((bKeysInBinChunk && type==COLORARRAY) || (!bKeysInBinChunk && type==COLOR));
			Color3fBegin(num_keys, fDuration);
			if(bKeysInBinChunk)
				for(int k=0; k<num_keys; k++)
				{
					pFileGeomData->Read(&x, sizeof(float));
					pFileGeomData->Read(&y, sizeof(float));
					pFileGeomData->Read(&z, sizeof(float));
					SetKey3(k, x, y, z);
				}
			else
				SetKey3(0, x, y, z);
			Color3fEnd();
			break;
		default:
			assert(false);
	}

	// verify location
	if(bKeysInBinChunk == true)
		assert((iGetCurPos+(int) uBSize) == pFileGeomData->GetCurPos());	// verify current location from chunk size

	return true;
}

bool CSceneCreatorParse::ParseQuats(const ASC_TOKEN type)
{
	match(type);
	match(LESS_THEN);
	assert(token.token_type==REAL_VALUE || token.token_type==STRING);
	
	float fDuration = -1;
	int32 num_keys, num_channels;
	float x, y, z, w;
	bool bKeysInBinChunk;
	int32 iGetCurPos=-1;
	uint32 uBSize=0;

	// array length longer then 1
	if(token.token_type==STRING)
	{
		// pickup chunk ID
		assert(token.str_len==4);
		const char cID[]={token.cptr[0], token.cptr[1], token.cptr[2], token.cptr[3], '\0'};
		match(STRING);
		bKeysInBinChunk = true;

		// pickup duration if one is found
		if(token.token_type==COMMA)
		{
			match(COMMA);
			assert(token.token_type == REAL_VALUE);
			fDuration = token_to_real();
			match(REAL_VALUE);
		}

		// locate chunk in binary file
		uBSize = LocateID(cID);	// lookup surface in geomdata
		iGetCurPos = pFileGeomData->GetCurPos();	

		// read num_keys and num_channels
		pFileGeomData->Read(&num_keys, sizeof(int32));
		pFileGeomData->Read(&num_channels, sizeof(int32));
		assert(num_channels == 4);
	}
	else	// otherwise pickup single key in file
	{
		bKeysInBinChunk = false;
		assert(token.token_type == REAL_VALUE);
		x = token_to_real();
		match(REAL_VALUE);
		match(COMMA);

		assert(token.token_type == REAL_VALUE);
		y = token_to_real();
		match(REAL_VALUE);
		match(COMMA);

		assert(token.token_type == REAL_VALUE);
		z = token_to_real();
		match(REAL_VALUE);
		match(COMMA);

		assert(token.token_type == REAL_VALUE);
		w = token_to_real();
		match(REAL_VALUE);
		
		num_keys = 1;
		num_channels = 4;
	}
	match(GREATER_THEN);


	// deliver keys
	switch(type)
	{
		case QUATERNION:	
		case QUATERNIONARRAY:
			assert((bKeysInBinChunk && type==QUATERNIONARRAY) || (!bKeysInBinChunk && type==QUATERNION));
			QuaternionBegin(num_keys, fDuration);
			if(bKeysInBinChunk)
				for(int k=0; k<num_keys; k++)
				{
					pFileGeomData->Read(&x, sizeof(float));
					pFileGeomData->Read(&y, sizeof(float));
					pFileGeomData->Read(&z, sizeof(float));
					pFileGeomData->Read(&w, sizeof(float));
					SetQuat(k, x, y, z, w);
				}
			else
				SetQuat(0, x, y, z, w);
			QuaternionEnd();
			break;
		default:
			assert(false);
	}

	// verify location
	if(bKeysInBinChunk == true)
		assert((iGetCurPos+(int) uBSize) == pFileGeomData->GetCurPos());	// verify current location from chunk size

	return true;

}

bool CSceneCreatorParse::ParseKeys1(const ASC_TOKEN type)
{
	match(type);
	match(LESS_THEN);
	assert(token.token_type==REAL_VALUE || token.token_type==STRING);
	
	float fDuration = -1;
	int32 num_keys, num_channels;
	float val;
	bool bKeysInBinChunk;
	int32 iGetCurPos=-1;
	uint32 uBSize=0;

	// array length longer then 1
	if(token.token_type==STRING)
	{
		// pickup chunk ID
		assert(token.str_len==4);
		const char cID[]={token.cptr[0], token.cptr[1], token.cptr[2], token.cptr[3], '\0'};
		match(STRING);
		bKeysInBinChunk = true;

		// pickup duration if one is found
		if(token.token_type==COMMA)
		{
			match(COMMA);
			assert(token.token_type == REAL_VALUE);
			fDuration = token_to_real();
			match(REAL_VALUE);
		}

		// locate chunk in binary file
		uBSize = LocateID(cID);	// lookup surface in geomdata
		iGetCurPos = pFileGeomData->GetCurPos();	

		// read num_keys and num_channels
		pFileGeomData->Read(&num_keys, sizeof(int32));
		pFileGeomData->Read(&num_channels, sizeof(int32));
		assert(num_channels == 1);
	}
	else	// otherwise pickup single key in file
	{
		bKeysInBinChunk = false;
		assert(token.token_type == REAL_VALUE);
		val = token_to_real();
		match(REAL_VALUE);
	
		num_keys = 1;
		num_channels = 1;
	}
	match(GREATER_THEN);


	// deliver keys
	switch(type)
	{
		case SCALE:
		case SCALEARRAY:
			assert((bKeysInBinChunk && type==SCALEARRAY) || (!bKeysInBinChunk && type==SCALE));
			ScaleBegin(num_keys, fDuration);
			FetchKeys1(num_keys, bKeysInBinChunk, val);
			ScaleEnd();
			break;
		case FIELDOFVIEW:
		case FIELDOFVIEWARRAY:
			assert((bKeysInBinChunk && type==FIELDOFVIEWARRAY) || (!bKeysInBinChunk && type==FIELDOFVIEW));
			FieldOfViewBegin(num_keys, fDuration);
			FetchKeys1(num_keys, bKeysInBinChunk, val);
			FieldOfViewEnd();
			break;
		case NEAR:
		case NEARARRAY:
			assert((bKeysInBinChunk && type==NEARARRAY) || (!bKeysInBinChunk && type==NEAR));
			NearBegin(num_keys, fDuration);
			FetchKeys1(num_keys, bKeysInBinChunk, val);
			NearEnd();
			break;
		case FAR:
		case FARARRAY:
			assert((bKeysInBinChunk && type==FARARRAY) || (!bKeysInBinChunk && type==FAR));
			FarBegin(num_keys, fDuration);
			FetchKeys1(num_keys, bKeysInBinChunk, val);
			FarEnd();
			break;
		case HOTSPOT:
		case HOTSPOTARRAY:
			assert((bKeysInBinChunk && type==HOTSPOTARRAY) || (!bKeysInBinChunk && type==HOTSPOT));
			SetSpot();
			HotSpotBegin(num_keys, fDuration);
			FetchKeys1(num_keys, bKeysInBinChunk, val);
			HotSpotEnd();
			break;
		case FALLOFF:
		case FALLOFFARRAY:
			assert((bKeysInBinChunk && type==FALLOFFARRAY) || (!bKeysInBinChunk && type==FALLOFF));
			SetSpot();
			FallOffBegin(num_keys, fDuration);
			FetchKeys1(num_keys, bKeysInBinChunk, val);
			FallOffEnd();
			break;
		case ASPECT:
		case ASPECTARRAY:
			assert((bKeysInBinChunk && type==ASPECTARRAY) || (!bKeysInBinChunk && type==ASPECT));
			SetSpot();
			AspectBegin(num_keys, fDuration);
			FetchKeys1(num_keys, bKeysInBinChunk, val);
			AspectEnd();
			break;
		default:
			assert(false);
	}

	// verify location
	if(bKeysInBinChunk == true)
		assert((iGetCurPos+(int) uBSize) == pFileGeomData->GetCurPos());	// verify current location from chunk size

	return true;
}

// anims special helper
void CSceneCreatorParse::FetchKeys1(const int32 num_keys, const bool bKeysInBinChunk, const float fDefVal)
{
	if(bKeysInBinChunk)
	for(int k=0; k<num_keys; k++)
	{
		float val;
		pFileGeomData->Read(&val, sizeof(float));
		SetKey1(k, val);
	}
	else
		SetKey1(0, fDefVal);
}

	
// just a small function assisting parse mesh by locating a surface in the bin geomfile
uint32 CSceneCreatorParse::LocateID(const char cID[])
{
	uint32 r=0;
	char buf[4];
	pFileGeomData->Read(buf, sizeof(int8)*4);

	// look for match in current location
	if(strncmp(buf, cID, 4) == 0)
		pFileGeomData->Read(&r, sizeof(uint32));
	else
	{
		pFileGeomData->LocateStart();
		int32 dataleft = (int32) pFileGeomData->Size();
		bool bFound=false;

		// surf for surface by ID
		while(!bFound && dataleft>0)
		{
			pFileGeomData->Read(buf, sizeof(int8)*4);
			bFound = strncmp(buf, cID, 4) == 0;
			pFileGeomData->Read(&r, sizeof(uint32));
			dataleft -= (r + sizeof(uint32) + sizeof(int8)*4);
			if(!bFound)
			{
				pFileGeomData->OffsetFromCur(r);	// step to next chunk
				r = 0;
			}
		}
	}

	return r;
}



/////////////////////////////////////////////////////////////////////////////////
///////////////////////////// lexical analysis //////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

char CSceneCreatorParse::token_strings[][MAX_TOKEN_STRING_LENGTH] =
{
	{"<"},
	{">"},
	{"{"},
	{"}"},
	{"["},
	{"]"},
	{","},
	{":"},
	{""},	// empty for string

	{""},	// empty for integer
	{""},	// empty for real_value

	{"animtime"},
	{"material"},
	{"mesh"},
	{"surface"},
	{"meshinstance"},
	{"meshref"},
	{"camera"},
	{"light"},

	{"position"},
	{"positionarray"},
	{"quaternion"},
	{"quaternionarray"},
	{"scale"},
	{"scalearray"},
	
	{"fieldofview"},
	{"fieldofviewarray"},
	{"ratio"},

	{"color"},
	{"colorarray"},
	{"near"},
	{"neararray"},
	{"far"},
	{"fararray"},
	{"hotspot"},
	{"hotspotarray"},
	{"falloff"},
	{"falloffarray"},
	{"aspect"},
	{"aspectarray"},
	{"spotshape"},
	{"CIRCULAR"},
	{"RECTANGULAR"},

	{"diffuse"},
	{"reflection"},
	{"bumpmap"},
	{"specular"}
};

// globals
int32 CSceneCreatorParse::szeScene = 0;
char * CSceneCreatorParse::pScene = NULL;
CSceneCreatorParse::Token CSceneCreatorParse::token;
IFileHandler * CSceneCreatorParse::pFileGeomData = NULL;
IFileHandler * CSceneCreatorParse::pFileScene = NULL;


bool CSceneCreatorParse::match(ASC_TOKEN type)
{
	assert(token.token_type == type);

	// get next token
	return fetch_token();
}

bool CSceneCreatorParse::fetch_token()
{
	bool res = false;

	// offset by previous string length, plus one to
	// remove '"' if last token was a string
	token.cptr += token.str_len;
	if(token.token_type == STRING)
		++token.cptr;

	// Remove whitespace
	RemoveWhiteSpace();

	// if we're left with any chars
	if(token.cptr < (pScene+szeScene))
	{
		if(token.cptr[0]=='"')	// check to see if this might be of type string
		{
			++token.cptr;	// skip '"'
			assert( token.cptr < (pScene+szeScene) && token.cptr[0]!='"');
			int n=0;
			while(token.cptr[n]!='"')
			{
				++n;
				assert( (token.cptr+n) < (pScene+szeScene));
			}
			
			token.str_len = n;
			token.token_type = STRING;
			res = true;
		}	// fetch integer or real_value
		else if(isdigit(token.cptr[0]) || (token.cptr[0]=='-' && (token.cptr+1<(pScene+szeScene)) && isdigit(token.cptr[1])))
		{
			int n=0;
			if( ((token.cptr+n) < (pScene+szeScene)) && token.cptr[n]=='-')
				++n;
			while( ((token.cptr+n) < (pScene+szeScene)) && isdigit(token.cptr[n]) )
				++n;

			// integer
			if( ((token.cptr+n) >= (pScene+szeScene)) || token.cptr[n]!='.')
			{
				token.str_len = n;
				token.token_type = INTEGER;
				res = true;
			}
			else	// real_value
			{
				++n;	// skip the dot
				assert( ((token.cptr+n) < (pScene+szeScene)) && isdigit(token.cptr[n]));
				while( ((token.cptr+n) < (pScene+szeScene)) && isdigit(token.cptr[n]) )
					++n;

				token.str_len = n;
				token.token_type = REAL_VALUE;
				res = true;
			}
		}
		else	// test all non-empty token strings for a match
		{
			int32 matchedIndex=-1, matchedLen = 0;
			for(int i=0; i<NUM_OF_ASC_TOKENS; i++)
			{
				// strings such as those of 
				// real_value and of the "symbol string"
				// are left empty
				int32 strLen = strlen(token_strings[i]);
				if(strLen>0)
				{
					bool bMatch = strncmp(token_strings[i], token.cptr, strLen) == 0;

					// is this a better match then
					// what we've found so far?
					if(bMatch && (matchedIndex==-1 || strLen>matchedLen))
					{
						matchedIndex = i;
						matchedLen = strLen;
					}
				}
			}

			if(matchedIndex==-1)
			{
				token.str_len = 1;
				token.token_type = UNKNOWN;
				res = true;
			}
			else
			{
				token.str_len = matchedLen;
				token.token_type = (ASC_TOKEN) matchedIndex;
				res = true;
			}
		}
	}

	return res;
}

int CSceneCreatorParse::token_to_integer()
{
	assert(token.token_type == INTEGER);

	int n=0; int res=0;
	if(n<token.str_len && token.cptr[n]=='-')
		++n;
	while(n<token.str_len && isdigit(token.cptr[n]))
	{

		res *= 10;
		res += (int) char_to_digit(token.cptr[n]);
		++n;
	}

	assert(n==token.str_len);
	return (token.cptr[0]=='-') ? -res : res;
}

float CSceneCreatorParse::token_to_real()
{
	assert(token.token_type == REAL_VALUE || token.token_type == INTEGER);

	int n=0; float res=0;
	if(n<token.str_len && token.cptr[n]=='-')
		++n;
	while(n<token.str_len && isdigit(token.cptr[n]))
	{

		res *= 10;
		res += (float) char_to_digit(token.cptr[n]);
		++n;
	}

	if(n!=token.str_len || token.token_type!=INTEGER)
	{
		++n;	// skip the dot
		assert(n<token.str_len && token.cptr[n-1]=='.' && token.token_type==REAL_VALUE);
		
		float div = 10.0f;
		while(n<token.str_len && isdigit(token.cptr[n]))
		{
			res += ((float) char_to_digit(token.cptr[n])/div);
			div *= 10.0f;
			++n;
		}
	}

	assert(n==token.str_len);
	return (token.cptr[0]=='-') ? -res : res;
}

int CSceneCreatorParse::char_to_digit(const char ch)
{
	// fast
	//return ch-'0';

	// safe
	int val;
	switch(ch)
	{
		case '0':
			val = 0;
			break;
		case '1':
			val = 1;
			break;
		case '2':
			val = 2;
			break;
		case '3':
			val = 3;
			break;
		case '4':
			val = 4;
			break;
		case '5':
			val = 5;
			break;
		case '6':
			val = 6;
			break;
		case '7':
			val = 7;
			break;
		case '8':
			val = 8;
			break;
		case '9':
			val = 9;
			break;
		default:
			assert(false);
	}

	// might do a safefast version using an array

	return val;
}


void CSceneCreatorParse::RemoveWhiteSpace()
{
	// remove
	do
	{
		if( (token.cptr+1<(pScene+szeScene)) && token.cptr[0]=='/' && token.cptr[1]=='/' )
			while( (token.cptr < (pScene+szeScene)) && token.cptr[0]!='\n' )
				++token.cptr;
		
		// remove classic whitespace
		while(
			  (token.cptr < (pScene+szeScene)) &&
			  (token.cptr[0]=='\r' || token.cptr[0]=='\t' || token.cptr[0]=='\n' || token.cptr[0]==' ')
			  )
			++token.cptr;
		
		
	}	// We done? (same as initial test)
	while( (token.cptr+1<(pScene+szeScene)) && token.cptr[0]=='/' && token.cptr[1]=='/' );
}
