#include "p2lassert.h"
#include "intmd/sceneintmd.h"
#include "intmd/textureintmd.h"


// material
void CSceneIntmd::IncMaterialBuffer(const char mat_name[])
{
	// add 1 to array
	if(m_pMaterials == NULL)
	{
		assert(GetNumMaterials()==0);
		m_pMaterials = new CMaterialIntmd[1];
	}
	else
	{
		CMaterialIntmd * pTmp = m_pMaterials;
		m_pMaterials = new CMaterialIntmd[GetNumMaterials() + 1];
		for(int m=0; m<GetNumMaterials(); m++)	// copy materials
			m_pMaterials[m] = pTmp[m];

		delete [] pTmp; pTmp = NULL;
	}

	GetMaterial(m_iNumMats).SetName(mat_name);
	++m_iNumMats;
}

void CSceneIntmd::delMaterial(const int32 num)
{
	// I'll just leave the space for the dead
	// material instead of reallocation
	for(int m=num; m<GetNumMaterials()-1; m++)
		m_pMaterials[m]=m_pMaterials[m+1];

	--m_iNumMats;
}

// mesh
void CSceneIntmd::IncMeshBuffer(const char mesh_name[])
{
	// add 1 to array
	if(m_pMeshes == NULL)
	{
		assert(GetNumMeshes()==0);
		m_pMeshes = new CMeshIntmd[1];
	}
	else
	{
		CMeshIntmd * pTmp = m_pMeshes;
		m_pMeshes = new CMeshIntmd[GetNumMeshes() + 1];
		for(int m=0; m<GetNumMeshes(); m++)	// copy meshes
			m_pMeshes[m] = pTmp[m];

		delete [] pTmp; pTmp = NULL;
	}

	GetMesh(m_iNumMeshes).SetName(mesh_name);
	++m_iNumMeshes;
}

void CMeshIntmd::SetName(const char name[])
{
	assert(m_pMeshName == NULL);
	int32 len = strlen(name) + 1;

	if((m_pMeshName=new char[len]) == NULL)
	{
		printf("allocation error\n");
		assert(false);
	}

	strcpy(m_pMeshName, name);
}

void CSceneIntmd::delMesh(const int32 num)
{
	// I'll just leave the space for the dead
	// mesh instead of reallocation
	for(int m=num; m<GetNumMeshes()-1; m++)
		m_pMeshes[m]=m_pMeshes[m+1];

	--m_iNumMeshes;
}

CTextureIntmd * CSceneIntmd::InsertIntoTexList( CTextureIntmd * pTexEntry, bool * pFoundMatch)
{
	// result var
	CTextureIntmd * res = NULL;
	
	// look for a match
	int i=0;
	bool bNotFound = true;
	while(i<m_iNumTextures && bNotFound)
	{
		if(*pTexEntry != m_pTextures[i])
			i++;
		else
			bNotFound = false;
	}

	// if we didn't find a match
	if(bNotFound)
	{
		// alloc another entry
		CTextureIntmd * pOldTex = m_pTextures;
		m_pTextures=NULL;
		m_pTextures=new CTextureIntmd[m_iNumTextures+1]; 
		assert(m_pTextures!=NULL);

		// copy old ones to current buffer
		for(int t=0; t<m_iNumTextures; t++)
			m_pTextures[t]=pOldTex[t];
		m_pTextures[m_iNumTextures] = *pTexEntry;
		res = &m_pTextures[m_iNumTextures++];
		if(pOldTex != NULL) { delete [] pOldTex; pOldTex = NULL; }
		*pFoundMatch = false;
	}
	else
	{
		res = &m_pTextures[i];
		*pFoundMatch = true;
	}
	
	// return result
	return res;
}

bool CMeshIntmd::operator !=(const CMeshIntmd &mesh)
{
	bool bRes = true;

	// compare all data
	if(strcmp(GetName(), mesh.GetName()) == 0)
	{
		if(GetNumSurfaces()==mesh.GetNumSurfaces())
		{	
			for(int s=0; s<GetNumSurfaces(); s++)
			{
				CSurfaceIntmd &surf = GetSurface(s);

				int k=0;
				bool bFoundSurf=false;
				while(k<GetNumSurfaces() && !bFoundSurf)
					bFoundSurf = !(surf != mesh.GetSurface(k++));

				bRes = bRes && bFoundSurf;
			}
		}
	}
	
	return bRes;
}

void CSceneIntmd::SwapMeshes(const int32 m1, const int32 m2)
{
	assert(m1>=0 && m1<GetNumMeshes());
	assert(m2>=0 && m2<GetNumMeshes());
	const CMeshIntmd tmp = m_pMeshes[m1];
	m_pMeshes[m1]=m_pMeshes[m2];
	m_pMeshes[m2]=tmp;
}

// surf
void CMeshIntmd::IncSurfBuffer(const int32 numverts, const int32 numstrips, const char matref[])
{
	// add 1 to array
	if(m_pSurfaces == NULL)
	{
		assert(GetNumSurfaces()==0);
		m_pSurfaces = new CSurfaceIntmd[1];
	}
	else
	{
		CSurfaceIntmd * pTmp = m_pSurfaces;
		m_pSurfaces = new CSurfaceIntmd[GetNumSurfaces() + 1];
		for(int m=0; m<GetNumSurfaces(); m++)	// copy meshes
			m_pSurfaces[m] = pTmp[m];

		delete [] pTmp; pTmp = NULL;
	}

	// setup data
	CSurfaceIntmd &surf = m_pSurfaces[GetNumSurfaces()];
	surf.Setup(numverts, numstrips, matref);	
		
	++m_iNumSurfs;
}

// meshinstance
void CSceneIntmd::IncMeshInstanceBuffer(const char meshinst_name[])
{
	// add 1 to array
	if(m_pMeshInstances == NULL)
	{
		assert(GetNumMeshInstances()==0);
		m_pMeshInstances = new CMeshInstanceIntmd[1];
	}
	else
	{
		CMeshInstanceIntmd * pTmp = m_pMeshInstances;
		m_pMeshInstances = new CMeshInstanceIntmd[GetNumMeshInstances() + 1];
		for(int m=0; m<GetNumMeshInstances(); m++)	// copy meshinstances
			m_pMeshInstances[m] = pTmp[m];

		delete [] pTmp; pTmp = NULL;
	}

	GetMeshInstance(m_iNumMeshInstances).SetName(meshinst_name);
	++m_iNumMeshInstances;
}

int32 CMeshInstanceIntmd::GetMeshIndex() const
{
	int32 index=-1;
	for(int m=0; m<m_pOwnerScene->GetNumMeshes(); m++)
	{
		if(strcmp(m_pMeshRef, m_pOwnerScene->GetMesh(m).GetName())==0)
		{
			assert(index == -1);	// no duplicates
			index = m;
		}
	}
	assert(index != -1);
	
	return index;
}

void CMeshInstanceIntmd::SetName(const char name[])
{
	assert(m_pMeshInstName == NULL);
	int32 len = strlen(name) + 1;

	if((m_pMeshInstName=new char[len]) == NULL)
	{
		printf("allocation error\n");
		assert(false);
	}

	strcpy(m_pMeshInstName, name);
}

void CMeshInstanceIntmd::SetMeshref(const char meshref[])
{
	m_pMeshRef = new char[strlen(meshref)+1];
	assert(m_pMeshRef != NULL);
	strcpy(m_pMeshRef, meshref);
}

void CSceneIntmd::SwapMeshInstances(const int32 m1, const int32 m2)
{
	assert(m1>=0 && m1<GetNumMeshInstances());
	assert(m2>=0 && m2<GetNumMeshInstances());
	const CMeshInstanceIntmd tmp = m_pMeshInstances[m1];
	m_pMeshInstances[m1]=m_pMeshInstances[m2];
	m_pMeshInstances[m2]=tmp;
}


// camera
void CSceneIntmd::IncCameraBuffer(const char cam_name[])
{
	// add 1 to array
	if(m_pCameras == NULL)
	{
		assert(GetNumCameras()==0);
		m_pCameras = new CCameraIntmd[1];
	}
	else
	{
		CCameraIntmd * pTmp = m_pCameras;
		m_pCameras = new CCameraIntmd[GetNumCameras() + 1];
		for(int m=0; m<GetNumCameras(); m++)	// copy cameras
			m_pCameras[m] = pTmp[m];

		delete [] pTmp; pTmp = NULL;
	}

	GetCamera(m_iNumCameras).SetName(cam_name);
	++m_iNumCameras;
}

void CCameraIntmd::SetName(const char name[])
{
	assert(m_pCamName == NULL);
	int32 len = strlen(name) + 1;

	if((m_pCamName=new char[len]) == NULL)
	{
		printf("allocation error\n");
		assert(false);
	}

	strcpy(m_pCamName, name);
}

// light
void CSceneIntmd::IncLightBuffer(const char lgt_name[])
{
	// add 1 to array
	if(m_pLights == NULL)
	{
		assert(GetNumLights()==0);
		m_pLights = new CLightIntmd[1];
	}
	else
	{
		CLightIntmd * pTmp = m_pLights;
		m_pLights = new CLightIntmd[GetNumLights() + 1];
		for(int m=0; m<GetNumLights(); m++)	// copy lights
			m_pLights[m] = pTmp[m];

		delete [] pTmp; pTmp = NULL;
	}

	GetLight(m_iNumLights).SetName(lgt_name);
	++m_iNumLights;
}

void CLightIntmd::SetName(const char name[])
{
	assert(m_pLgtName == NULL);
	int32 len = strlen(name) + 1;

	if((m_pLgtName=new char[len]) == NULL)
	{
		printf("allocation error\n");
		assert(false);
	}

	strcpy(m_pLgtName, name);
}

// the scene cleanup crew
void CSceneIntmd::CleanUp()
{
	// handle textures
	CleanUpTextures();
	
	// handle materials
	for(int m=0; m<m_iNumMats; m++)
		m_pMaterials[m].CleanUp();

	// handle meshes
	CleanUpMeshes();
	
	// handle meshinstances
	for(int m=0; m<m_iNumMeshInstances; m++)
		m_pMeshInstances[m].CleanUp();

	// handle cameras
	for(int c=0; c<m_iNumCameras; c++)
		m_pCameras[c].CleanUp();

	// handle lights
	for(int l=0; l<m_iNumLights; l++)
		m_pLights[l].CleanUp();
	
	// finish up
	m_iNumMeshInstances=0;
	m_iNumLights=0;
	m_iNumCameras=0;
	m_iNumNullMeshes=0;
	m_iNumNullMeshInstances=0;
	m_iNumMats=0;	
	
	if(m_pMeshInstances!=NULL) { delete [] m_pMeshInstances; m_pMeshInstances=NULL; }
	if(m_pLights!=NULL) { delete [] m_pLights; m_pLights=NULL; }
	if(m_pCameras!=NULL) { delete [] m_pCameras; m_pCameras=NULL; }
	if(m_pMaterials!=NULL) { delete [] m_pMaterials; m_pMaterials=NULL; }
}

void CSceneIntmd::CleanUpMeshes()
{
	for(int m=0; m<m_iNumMeshes; m++)
		m_pMeshes[m].CleanUp();

	m_iNumMeshes=0;
	if(m_pMeshes!=NULL) { delete [] m_pMeshes; m_pMeshes=NULL; }
}

void CSceneIntmd::CleanUpTextures()
{
	for(int t=0; t<m_iNumTextures; t++)
		m_pTextures[t].CleanUp();

	m_iNumTextures=0;
	if(m_pTextures!=NULL) { delete [] m_pTextures; m_pTextures=NULL; }
}

void CMeshIntmd::CleanUp()
{
	// erase surfaces
	for(int s=0; s<m_iNumSurfs; s++)
		m_pSurfaces[s].CleanUp();
	if(m_pSurfaces!=NULL) { delete [] m_pSurfaces; m_pSurfaces=NULL; }
	m_iNumSurfs=0;
	
	// erase meshname
	if(m_pMeshName!=NULL) { delete [] m_pMeshName; m_pMeshName=NULL; }
};

void CMeshInstanceIntmd::CleanUp()
{
	// just a ref, mesh has
	// already been deleted
	if(m_pMeshRef!=NULL) { delete [] m_pMeshRef; m_pMeshRef=NULL; }

	// erase instance name
	if(m_pMeshInstName!=NULL) { delete [] m_pMeshInstName; m_pMeshInstName=NULL; }
		
	// erase transformation
	m_vPosArray.CleanUp();
	m_qRotArray.CleanUp();
	m_fUniScaleArray.CleanUp();
};

void CLightIntmd::CleanUp()
{
	// erase light name
	if(m_pLgtName!=NULL) { delete [] m_pLgtName; m_pLgtName=NULL; }

	// erase transformation
	m_vPosArray.CleanUp();
	m_qRotArray.CleanUp();
	m_fUniScaleArray.CleanUp();

	// erase light specific animation props
	m_vColorArray.CleanUp();
	m_fNearArray.CleanUp();
	m_fFarArray.CleanUp();
	m_fHotSpotArray.CleanUp();
	m_fFallOffArray.CleanUp();
	m_fAspectArray.CleanUp();

	// init variables
	m_bIsSpot=false;
	m_bIsRectangular=false;
};

void CCameraIntmd::CleanUp()
{
	// erase light name
	if(m_pCamName!=NULL) { delete [] m_pCamName; m_pCamName=NULL; }

	// erase transformation
	m_vPosArray.CleanUp();
	m_qRotArray.CleanUp();
	m_fUniScaleArray.CleanUp();

	// erase camera specific animation props
	m_fFovArray.CleanUp();
	
	// init variables
	m_fAspect = 1.0f;
};

