#include "intmd/textureintmd.h"
#include "intmd/pixel_formats.h"
#include "p2lassert.h"




bool CTextureIntmd::operator !=(const CTextureIntmd &tex)
{
	bool res = true;

	if(strcmp(GetUserDefProps(), tex.GetUserDefProps()) == 0)
	{
		if(GetPixelFormat() == tex.GetPixelFormat())
		{
			if(GetNumLevels() == tex.GetNumLevels())
			{
				// compare palettes if any were found
				const int32 num_cols4 = 4*((GetPixelFormat()==INDEXCOLOR16) ? 16 : ((GetPixelFormat()==INDEXCOLOR256) ? 256 : 0));
				bool bIsMatch = true;
				int i=0;
				const uint8 * pal1 = (const uint8 *) GetPalette();
				const uint8 * pal2 = (const uint8 *) tex.GetPalette();
				while(i<num_cols4 && bIsMatch)
				{
					bIsMatch = pal1[i]==pal2[i];
					++i;
				}

				if(bIsMatch)
				{
					int m=0;
					while(m<GetNumLevels() && bIsMatch)
					{
						// check levels
						const CBinaryBitmapIntmd &bmap1 = GetLevel(m);
						const CBinaryBitmapIntmd &bmap2 = tex.GetLevel(m);

						if(bmap1.GetWidth()==bmap2.GetWidth() && bmap1.GetHeight()==bmap2.GetHeight())
						{
							const int32 pix_size = (GetPixelFormat()==INDEXCOLOR16) ? 1 : ((GetPixelFormat()==INDEXCOLOR256) ? 1 : 4);
							const int32 num_bytes = bmap1.GetWidth()*bmap1.GetHeight()*pix_size;
							const uint8 * ptr1 = (const uint8 *)bmap1.GetTexelData();
							const uint8 * ptr2 = (const uint8 *)bmap2.GetTexelData();

							int b=0;
							while(b<num_bytes && bIsMatch)
							{
								bIsMatch = ptr1[b]==ptr2[b];
								++b;
							}
						}
						else bIsMatch=false;
						
						// next
						++m;
					}

					res = !bIsMatch;
				}
			}
		}
	}
	
	return res;
}



bool CTextureSeqIntmd::operator !=(const CTextureSeqIntmd &texseq) const
{
	bool res = true;

	if(m_fDuration==texseq.GetDuration() && m_iNumTexFrames==texseq.GetNumTexFrames())
	{
		int i=0;
		while(i<m_iNumTexFrames && res)			
		{
			res = GetTexFrameIndex(i)==texseq.GetTexFrameIndex(i);

			// next
			++i;
		}
	}

	return res;
}

void CTextureSeqIntmd::IncTexBuffer(const int32 iNewTexFrameEntry)
{
	// add 1 to array
	if(m_pTexFrames == NULL)
	{
		assert(GetNumTexFrames()==0);
		m_pTexFrames = new int32[1];
	}
	else
	{
		int32 * pTmp = m_pTexFrames;
		m_pTexFrames = NULL;
		m_pTexFrames = new int32[GetNumTexFrames() + 1];
		assert(m_pTexFrames!=NULL);
		for(int t=0; t<GetNumTexFrames(); t++)	// copy texture pointers
			m_pTexFrames[t] = pTmp[t];
	
		delete [] pTmp; pTmp = NULL;
	}

	// finish off
	m_pTexFrames[GetNumTexFrames()] = iNewTexFrameEntry;
	++m_iNumTexFrames;
}


// clean up
void CTextureIntmd::CleanUp()
{
	// erase pixeldata
	for(int m=0; m<m_iNumLevels; m++)
		if(m_pMipMaps[m].m_pTexelData!=NULL)
			delete [] m_pMipMaps[m].m_pTexelData;
	delete [] m_pMipMaps; m_pMipMaps=NULL;

	// erase UserProps
	if(m_pUserDefProps!=NULL)
	{ delete [] m_pUserDefProps; m_pUserDefProps=NULL; }

	// erase palette if any
	if(m_pInternPalette!=NULL)
	{ delete [] m_pInternPalette; m_pInternPalette=NULL; }

	// set integer members
	m_iPixelFormat=-1;
	m_iNumLevels=0;
	m_uChkID=0;	// ID chunk from file
}


void CTextureSeqIntmd::CleanUp()
{
	m_fDuration = -1.0f;
	m_iNumTexFrames = 0;
	if(m_pTexFrames!=NULL) { delete [] m_pTexFrames; m_pTexFrames=NULL; }
}
